/***********************************************************
 * This software is part of the ProM package * http://www.processmining.org/ * *
 * Copyright (c) 2003-2006 TU/e Eindhoven * and is licensed under the * Common
 * Public License, Version 1.0 * by Eindhoven University of Technology *
 * Department of Information Systems * http://is.tm.tue.nl * *
 **********************************************************/

package org.processmining.plugins.DeclareConformance;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.ltl2automaton.plugins.automaton.Automaton;
import org.processmining.ltl2automaton.plugins.automaton.Transition;
import org.processmining.ltl2automaton.plugins.formula.DefaultParser;
import org.processmining.ltl2automaton.plugins.formula.Formula;
import org.processmining.ltl2automaton.plugins.formula.conjunction.ConjunctionFactory;
import org.processmining.ltl2automaton.plugins.formula.conjunction.ConjunctionTreeLeaf;
import org.processmining.ltl2automaton.plugins.formula.conjunction.ConjunctionTreeNode;
import org.processmining.ltl2automaton.plugins.formula.conjunction.DefaultTreeFactory;
import org.processmining.ltl2automaton.plugins.formula.conjunction.GroupedTreeConjunction;
import org.processmining.ltl2automaton.plugins.formula.conjunction.TreeFactory;
import org.processmining.ltl2automaton.plugins.ltl.SyntaxParserException;
import org.processmining.plugins.DataConformance.Alignment;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.declare.visualizing.ActivityDefinition;
import org.processmining.plugins.declare.visualizing.AssignmentModel;
import org.processmining.plugins.declare.visualizing.AssignmentModelView;
import org.processmining.plugins.declare.visualizing.ConstraintDefinition;
import org.processmining.plugins.declare.visualizing.Parameter;
import org.processmining.plugins.declareminer.ExecutableAutomaton;
import org.processmining.plugins.declareminer.PossibleNodes;
import org.processmining.plugins.declareminer.visualizing.DeclareMap;


@Plugin(name = "Declare Diagnoser", parameterLabels = {"Replay" }, returnLabels = { "Diagnosis Results" }, returnTypes = { DeclareCheckerViz.class }, userAccessible = true)
public class DeclareChecker {

	HashMap frequencies;

	//Progress prog;

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "M. de Leoni, F.M. Maggi", email = "m.d.leoni@tue.nl, F.M.Maggi@tue.nl", uiLabel = "Declare Diagnoser", pack = "DeclareChecker")
	@PluginVariant(requiredParameterLabels = { 0 }, variantLabel = "Declare Diagnoser")
	public DeclareCheckerViz analyse(UIPluginContext context,  ResultReplayDeclare replay) {
		DeclareCheckerViz outcome = declareMinerFrame(false, replay, context);
		//prog.setValue(112);
		return outcome;
	}

	public DeclareCheckerViz declareMinerFrame(boolean console, ResultReplayDeclare replay, UIPluginContext context) {
		DeclareMap declaremodel = replay.getDeclareModel();
		//	DeclareModel output = new DeclareModel((AssignmentModel)model.getModel().clone(), new AssignmentModelView(model.getModel()));
		//HashMap activityDefsValues = new HashMap();
		AssignmentModel model = declaremodel.getBrokerCh().readAssignment();
		AssignmentModelView view = new AssignmentModelView(model);
		declaremodel.getBrokerCh().readAssignmentGraphical(model, view);
		DeclareMap output = new DeclareMap(null,model,null, view,null,declaremodel.getBrokerCh());
		Collection<Alignment> alignments = replay.labelStepArray;
		HashMap adML = new HashMap();
		HashMap adMP = new HashMap();
		HashMap cdML = new HashMap();
		HashMap cdMP = new HashMap();
		HashMap adMLP = new HashMap();
		//int n = 0;
		context.getProgress().setMaximum(output.getModelCh().activityDefinitionsCount()+output.getModelCh().constraintDefinitionsCount());

		for(ActivityDefinition ad: output.getModelCh().getActivityDefinitions()){
			int ml = 0;
			int mp = 0;
			int mlp = 0;
			context.getProgress().inc();

			for(Alignment al:alignments){
				ml = ml + getMoveInLog(ad, al.getProcessTrace(), al.getLogTrace());
				mp = mp + getMoveInProcess(ad, al.getProcessTrace(), al.getLogTrace());
				mlp = mlp + getMoveInLogInProcess(ad, al.getProcessTrace(), al.getLogTrace());

			}	

			//output.getView().setActivityDefinitionLabel(ad, ml+"/"+mp+"/"+mlp);
			if(ml==0 && mp==0 && mlp==0){
				output.getViewCh().setActivityDefinitionLabel(ad, "-");
				Color bg = Color.white;
				output.getViewCh().setActivityDefinitionBackground(ad, bg);
			}else{
				double value=(mlp/((double)mp+(double)ml+mlp));
				output.getViewCh().setActivityDefinitionLabel(ad, ""+Approssimator.approssimate(value,3));
				double red,green;
				if (value<0.5)
				{
					red=255;
					green=125*2*value;
				}
				else
				{
					green=100;
					//green=125;
					red=255*2*(1-value);
				}		
				Color bg = new Color((int)red, (int)green,0);
				output.getViewCh().setActivityDefinitionBackground(ad, bg);
			}
			adML.put(ad.getId(), ml);
			adMP.put(ad.getId(), mp);
			adMLP.put(ad.getId(), mlp);
		}
		double current = 0;
		for(ConstraintDefinition cd: output.getModelCh().getConstraintDefinitions()){
			context.getProgress().inc();

			for(ActivityDefinition ad: output.getModelCh().getActivityDefinitions()){
				int skips = 0;
				int adds = 0;	
				for(Alignment al:alignments){
					int skipped = skippedActivityDefinition(al.getLogTrace(), al.getProcessTrace(), cd, ad);
					int added =  addedActivityDefinition(al.getLogTrace(), al.getProcessTrace(), cd, ad);
					skips = skips + skipped;
					adds = adds + added; 
				}
				cdMP.put(cd.getId()+"@"+ad.getName(), adds);
				cdML.put(cd.getId()+"@"+ad.getName(), skips);
			}
			double n = 0;
			double label = 0;
			double percentage = 0.;
			double num = 0.;
			for(Alignment al:alignments){
				num++;
				int skips = 0;
				int adds = 0;	
				int skipped = 0;
				int added = 0;
				for(ActivityDefinition ad: output.getModelCh().getActivityDefinitions()){
					skipped = skippedActivityDefinition(al.getLogTrace(), al.getProcessTrace(), cd, ad);
					added =  addedActivityDefinition(al.getLogTrace(), al.getProcessTrace(), cd, ad);
					skips = skips + skipped;
					adds = adds + added; 
				}
				if(skips>0 || adds >0){
					percentage ++;
				}
				current = 1.- (((double)skips + (double)adds)/al.getProcessTrace().size()); //both = both + (al.getProcessTrace().size() - (skipped + added));

				label = label + current;
				n ++;
			}

			//if(  ((double)totalboth==0) && ((double)totalboth+(double)totalskips+(double)totaladds)==0){
			//	cd.setDisplay("-");	
			//}else{
			//cd.setDisplay(labelskip+"\n"+labeladd);
			double met = label/n;

			double perInt = percentage/num;

			double perc = Approssimator.approssimate(perInt,7);

			//if(met < 0.976 && !(met < 0.946)){
			//	met = met - 0.5;
			//}else if(met < 0.946){
			//	met = met - 0.9;
			//}else if(met < 1){
			//	met = met - 0.2;
			//	}
			cd.setDisplay(""+Approssimator.approssimate(met,7)+"-"+perc);
			double red,green;
			if (met<0.5)
			{
				red=255;
				green=125*2*met;
			}
			else
			{
				green=125;
				red=255*2*(1-met);
			}				
			Color bg = new Color((int)red, (int)green,0);

			//}
		//view.setConstraintDefinitionColor(cd, Color.black, Color.white, Color.white);
			view.setConstraintDefinitionColor(cd, bg, Color.black, Color.gray);
			//view.setConstraintDefinitionLabel(cd, labelskip+"@"+labeladd);
		}
		DeclareCheckerViz dcv = new DeclareCheckerViz();
		dcv.setModel(output);
		dcv.setAdML(adML);
		dcv.setAdMLP(adMLP);
		dcv.setAdMP(adMP);
		dcv.setCdML(cdML);
		dcv.setCdMP(cdMP);
		//out.getView().setActivityDefinitionBackground(out.getModel().getActivityDefinitions().iterator().next(),
		//	Color.YELLOW);
		//out.getView().setActivityDefinitionLabel(out.getModel().getActivityDefinitions().iterator().next(), "0/1/1");
		return dcv;
	}


	public int getMoveInLog(ActivityDefinition ad, List<ExecutionStep> processTrace, List<ExecutionStep> logTrace){
		int s = 0;
		int output = 0;
		for (ExecutionStep step : processTrace) {
			if (step.equals(ExecutionStep.bottomStep) && !logTrace.get(s).equals(ExecutionStep.bottomStep)) {
				if(logTrace.get(s).getActivity().equals(ad.getName())){
					output++;
				}
			}
			s++;
		}
		return output;
	}

	public int getMoveInProcess(ActivityDefinition ad, List<ExecutionStep> processTrace, List<ExecutionStep> logTrace){
		int s = 0;
		int output = 0;
		for (ExecutionStep step : processTrace) {
			if (!step.equals(ExecutionStep.bottomStep) && logTrace.get(s).equals(ExecutionStep.bottomStep)) {
				if(step.getActivity().equals(ad.getName())){
					output++;
				}
			}
			s++;
		}
		return output;
	}

	public int getMoveInLogInProcess(ActivityDefinition ad, List<ExecutionStep> processTrace, List<ExecutionStep> logTrace){
		int s = 0;
		int output = 0;
		for (ExecutionStep step : processTrace) {
			if (!step.equals(ExecutionStep.bottomStep) && !logTrace.get(s).equals(ExecutionStep.bottomStep)) {
				if(logTrace.get(s).getActivity().equals(ad.getName())){
					output++;
				}
			}
			s++;
		}
		return output;
	}




	public int skippedActivityDefinition(List<ExecutionStep> logTrace, List<ExecutionStep> processTrace, ConstraintDefinition cd, ActivityDefinition ad) {
		String[] skipped = new String[logTrace.size()];
		String	constraint = cd.getName();
		int count = 1;
		constraint = constraint + "(";
		for (Parameter p : cd.getParameters()) {
			if (count < cd.parameterCount()) {
				int countB = 1;
				constraint = constraint + "[";
				for (ActivityDefinition b : cd.getBranches(p)) {
					if (countB < cd.branchesCount(p)) {
						constraint = constraint + b.getName() + ",";
					} else {
						constraint = constraint + b.getName() + "],";
					}
					countB++;
				}
			} else {
				int countB = 1;
				constraint = constraint + "[";
				for (ActivityDefinition b : cd.getBranches(p)) {
					if (countB < cd.branchesCount(p)) {
						constraint = constraint + b.getName() + ",";
					} else {
						constraint = constraint + b.getName() + "])";
					}
					countB++;
				}
			}
			count++;
		}
		String formula = cd.getText();

		if (cd.getName().equals("absence")){
			formula = "!( <> ( \"A\" ) )";
		}
		if (cd.getName().equals("absence2")){
			formula = "! ( <> ( ( \"A\" /\\ X(<>(\"A\")) ) ) )";
		}
		if (cd.getName().equals("absence3")){
			formula = "! ( <> ( ( \"A\" /\\  X ( <> ((\"A\" /\\  X ( <> ( \"A\" ) )) ) ) ) ))";
		}
		
		if (cd.getName().equals("alternate precedence")){
			formula = "(((( !(\"B\") U \"A\") \\/ []( !(\"B\"))) /\\ []((\"B\" ->( (!(X(\"A\")) /\\ !(X(!(\"A\"))) ) \\/ X((( !(\"B\") U \"A\") \\/ []( !(\"B\")))))))) /\\ !(\"B\"))";
		}
		if (cd.getName().equals("alternate response")){
			formula = "( []( ( \"A\" -> X(( (! ( \"A\" )) U \"B\" ) )) ) )";
		}
		if (cd.getName().equals("alternate succession")){
			formula = "( []((\"A\" -> X(( !(\"A\") U \"B\")))) /\\ (((( !(\"B\") U \"A\") \\/ []( !(\"B\"))) /\\ []((\"B\" ->( (!(X(\"A\")) /\\ !(X(!(\"A\"))) ) \\/ X((( !(\"B\") U \"A\") \\/ []( !(\"B\")))))))) /\\ !(\"B\")))";
		}
		if (cd.getName().equals("chain precedence")){
			formula = "[]( ( X( \"B\" ) -> \"A\") )/\\ ! (\"B\" )";
		}
		if (cd.getName().equals("chain response")){
			formula = "[] ( ( \"A\" -> X( \"B\" ) ) )";
		}
		if (cd.getName().equals("chain succession")){
			formula = "([]( ( \"A\" -> X( \"B\" ) ) )) /\\ ([]( ( X( \"B\" ) ->  \"A\") ) /\\ ! (\"B\" ))";
		}
		if (cd.getName().contains("of")){
			formula = "(  <> ( \"A\" ) \\/ <>( \"B\" )  )";
		}
		if (cd.getName().equals("co-existence")){
			formula = "( ( <>(\"A\") -> <>( \"B\" ) ) /\\ ( <>(\"B\") -> <>( \"A\" ) )  )";
		}
		if (cd.getName().equals("exactly1")){
			formula = "(  <> (\"A\") /\\ ! ( <> ( ( \"A\" /\\ X(<>(\"A\")) ) ) ) )";
		}
	
		if (cd.getName().equals("exactly2")){
			formula = "( <> (\"A\" /\\ (\"A\" -> (X(<>(\"A\"))))) /\\  ! ( <>( \"A\" /\\ (\"A\" -> X( <>( \"A\" /\\ (\"A\" -> X ( <> ( \"A\" ) ))) ) ) ) ) )";
		}
		if (cd.getName().equals("exclusive choice")){
			formula = "(  ( <>( \"A\" ) \\/ <>( \"B\" )  )  /\\ !( (  <>( \"A\" ) /\\ <>( \"B\" ) ) ) )";
		}
		if (cd.getName().equals("existence")){
			formula = "( <> ( \"A\" ) )";
		}
		if (cd.getName().equals("existence2")){
			formula = "<> ( ( \"A\" /\\ X(<>(\"A\")) ) )";
		}
		if (cd.getName().equals("existence3")){
			formula = "<>( \"A\" /\\ X(  <>( \"A\" /\\ X( <> \"A\" )) ))";
		}
		if (cd.getName().equals("strong init")){
			formula = "( \"A\" )";
		}
		if (cd.getName().equals("init")){
			formula = "( \"A\" )";
		}
		if (cd.getName().equals("not chain succession")){
			formula = "[]( ( \"A\" -> !(X( \"B\" ) ) ))";
		}
		if (cd.getName().equals("not co-existence")){
			formula = "(<>(\"A\")) -> (!(<>( \"B\" )))";
		}
		if (cd.getName().equals("not succession")){
			formula = "[]( ( \"A\" -> !(<>( \"B\" ) ) ))";
		}
		if (cd.getName().equals("precedence")){
			formula = "( ! (\"B\" ) U \"A\" ) \\/ ([](!(\"B\"))) /\\ ! (\"B\" )";
		}
		if (cd.getName().equals("responded existence")){
			formula = "(( ( <>( \"A\" ) -> (<>( \"B\" ) )) ))";
		}
		if (cd.getName().equals("response")){
			formula = "( []( ( \"A\" -> <>( \"B\" ) ) ))";
		}
		if (cd.getName().equals("succession")){
			formula = "(( []( ( \"A\" -> <>( \"B\" ) ) ))) /\\ (( ! (\"B\" ) U \"A\" ) \\/ ([](!(\"B\"))) /\\ ! (\"B\" ))";
		}

		for (Parameter p : cd.getParameters()) {
			int countB = 1;
			String param = null;
			String actualParameter = "(\"";
			if(cd.getBranches(p).size()==0){actualParameter = actualParameter + "EMPTY_PARAM" + "\")";}
			for (ActivityDefinition b : cd.getBranches(p)) {
				String bname = b.getName();
				if(bname == null || bname.equals("") ){
					bname = "EMPTY_PARAM";
				}
				if (countB < cd.branchesCount(p)) {
					actualParameter = actualParameter + b.getName() + "\"||\"";
				} else {
					actualParameter = actualParameter + b.getName() + "\")";
					param = b.getName();
				}
				countB++;
			}
			formula = formula.replace("\"" + p.getName() + "\"", actualParameter);

		}
		formula = formula.replace("/\\ event==COMPLETE", "");
		formula = formula.replace("/\\ event==complete", "");
		formula = formula.replace("activity==", "");
		formula = formula.replace("_O", "X");
		formula = formula.replace("U_", "U");



		Automaton pAut = new Automaton();
		List<Formula> formulaeParsed = new ArrayList<Formula>();
		try {
			formulaeParsed.add(new DefaultParser(formula).parse());
		} catch (SyntaxParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		TreeFactory<ConjunctionTreeNode, ConjunctionTreeLeaf> treeFactory = DefaultTreeFactory.getInstance();
		ConjunctionFactory<? extends GroupedTreeConjunction> conjunctionFactory = GroupedTreeConjunction
				.getFactory(treeFactory);
		GroupedTreeConjunction conjunction = conjunctionFactory.instance(formulaeParsed);
		pAut = conjunction.getAutomaton().op.reduce();
		ExecutableAutomaton pExecAut = new ExecutableAutomaton(pAut);
		int s = 0;
		for (ExecutionStep step : processTrace) {
			if (step.equals(ExecutionStep.bottomStep) && !logTrace.get(s).equals(ExecutionStep.bottomStep)) {
				if(logTrace.get(s).getActivity().equals(ad.getName())){
					skipped[s] = logTrace.get(s).getActivity();
				}
			}
			s++;
		}
		int output = 0;
		for (int d = 0; d < skipped.length; d++) {
			if (skipped[d] != null) {
				int t = 0;
				for (ExecutionStep step : processTrace) {
					if (!step.equals(ExecutionStep.bottomStep) || (d == t)) {
						String label = null;
						if (!step.equals(ExecutionStep.bottomStep)) {
							label = step.getActivity();
						}
						if (d == t) {
							label = logTrace.get(t).getActivity();
						}
						boolean violated = true;
						PossibleNodes current = null;
						current = pExecAut.currentState();
						for (Transition out : current.output()) {
							if (out.parses(label)) {
								violated = false;
								break;
							}
						}
						if (!violated) {
							pExecAut.next(label);
						} else {
							output ++;
							break;
						}
					}
					t++;
				}
				if(!pExecAut.currentState().isAccepting()){
					output ++;
				}
				pExecAut.ini();
			}			
		}
		return output;
	}


	public int addedActivityDefinition(List<ExecutionStep> logTrace, List<ExecutionStep> processTrace, ConstraintDefinition cd, ActivityDefinition ad) {
		String[] added = new String[logTrace.size()];
		String	constraint = cd.getName();
		int count = 1;
		constraint = constraint + "(";
		for (Parameter p : cd.getParameters()) {
			if (count < cd.parameterCount()) {
				int countB = 1;
				constraint = constraint + "[";
				for (ActivityDefinition b : cd.getBranches(p)) {
					if (countB < cd.branchesCount(p)) {
						constraint = constraint + b.getName() + ",";
					} else {
						constraint = constraint + b.getName() + "],";
					}
					countB++;
				}
			} else {
				int countB = 1;
				constraint = constraint + "[";
				for (ActivityDefinition b : cd.getBranches(p)) {
					if (countB < cd.branchesCount(p)) {
						constraint = constraint + b.getName() + ",";
					} else {
						constraint = constraint + b.getName() + "])";
					}
					countB++;
				}
			}
			count++;
		}
		String formula = cd.getText();

		if (cd.getName().equals("absence")){
			formula = "!( <> ( \"A\" ) )";
		}
		if (cd.getName().equals("absence2")){
			formula = "! ( <> ( ( \"A\" /\\ X(<>(\"A\")) ) ) )";
		}
		if (cd.getName().equals("absence3")){
			formula = "! ( <> ( ( \"A\" /\\  X ( <> ((\"A\" /\\  X ( <> ( \"A\" ) )) ) ) ) ))";
		}
		
		if (cd.getName().equals("alternate precedence")){
			formula = "(((( !(\"B\") U \"A\") \\/ []( !(\"B\"))) /\\ []((\"B\" ->( (!(X(\"A\")) /\\ !(X(!(\"A\"))) ) \\/ X((( !(\"B\") U \"A\") \\/ []( !(\"B\")))))))) /\\ !(\"B\"))";
		}
		if (cd.getName().equals("alternate response")){
			formula = "( []( ( \"A\" -> X(( (! ( \"A\" )) U \"B\" ) )) ) )";
		}
		if (cd.getName().equals("alternate succession")){
			formula = "( []((\"A\" -> X(( !(\"A\") U \"B\")))) /\\ (((( !(\"B\") U \"A\") \\/ []( !(\"B\"))) /\\ []((\"B\" ->( (!(X(\"A\")) /\\ !(X(!(\"A\"))) ) \\/ X((( !(\"B\") U \"A\") \\/ []( !(\"B\")))))))) /\\ !(\"B\")))";
		}
		if (cd.getName().equals("chain precedence")){
			formula = "[]( ( X( \"B\" ) -> \"A\") )/\\ ! (\"B\" )";
		}
		if (cd.getName().equals("chain response")){
			formula = "[] ( ( \"A\" -> X( \"B\" ) ) )";
		}
		if (cd.getName().equals("chain succession")){
			formula = "([]( ( \"A\" -> X( \"B\" ) ) )) /\\ ([]( ( X( \"B\" ) ->  \"A\") ) /\\ ! (\"B\" ))";
		}
		if (cd.getName().contains("of")){
			formula = "(  <> ( \"A\" ) \\/ <>( \"B\" )  )";
		}
		if (cd.getName().equals("co-existence")){
			formula = "( ( <>(\"A\") -> <>( \"B\" ) ) /\\ ( <>(\"B\") -> <>( \"A\" ) )  )";
		}
		if (cd.getName().equals("exactly1")){
			formula = "(  <> (\"A\") /\\ ! ( <> ( ( \"A\" /\\ X(<>(\"A\")) ) ) ) )";
		}
	
		if (cd.getName().equals("exactly2")){
			formula = "( <> (\"A\" /\\ (\"A\" -> (X(<>(\"A\"))))) /\\  ! ( <>( \"A\" /\\ (\"A\" -> X( <>( \"A\" /\\ (\"A\" -> X ( <> ( \"A\" ) ))) ) ) ) ) )";
		}
		if (cd.getName().equals("exclusive choice")){
			formula = "(  ( <>( \"A\" ) \\/ <>( \"B\" )  )  /\\ !( (  <>( \"A\" ) /\\ <>( \"B\" ) ) ) )";
		}
		if (cd.getName().equals("existence")){
			formula = "( <> ( \"A\" ) )";
		}
		if (cd.getName().equals("existence2")){
			formula = "<> ( ( \"A\" /\\ X(<>(\"A\")) ) )";
		}
		if (cd.getName().equals("existence3")){
			formula = "<>( \"A\" /\\ X(  <>( \"A\" /\\ X( <> \"A\" )) ))";
		}
		if (cd.getName().equals("strong init")){
			formula = "( \"A\" )";
		}
		if (cd.getName().equals("init")){
			formula = "( \"A\" )";
		}
		if (cd.getName().equals("not chain succession")){
			formula = "[]( ( \"A\" -> !(X( \"B\" ) ) ))";
		}
		if (cd.getName().equals("not co-existence")){
			formula = "(<>(\"A\")) -> (!(<>( \"B\" )))";
		}
		if (cd.getName().equals("not succession")){
			formula = "[]( ( \"A\" -> !(<>( \"B\" ) ) ))";
		}
		if (cd.getName().equals("precedence")){
			formula = "( ! (\"B\" ) U \"A\" ) \\/ ([](!(\"B\"))) /\\ ! (\"B\" )";
		}
		if (cd.getName().equals("responded existence")){
			formula = "(( ( <>( \"A\" ) -> (<>( \"B\" ) )) ))";
		}
		if (cd.getName().equals("response")){
			formula = "( []( ( \"A\" -> <>( \"B\" ) ) ))";
		}
		if (cd.getName().equals("succession")){
			formula = "(( []( ( \"A\" -> <>( \"B\" ) ) ))) /\\ (( ! (\"B\" ) U \"A\" ) \\/ ([](!(\"B\"))) /\\ ! (\"B\" ))";
		}

		for (Parameter p : cd.getParameters()) {
			int countB = 1;
			String param = null;
			String actualParameter = "(\"";
			if(cd.getBranches(p).size()==0){actualParameter = actualParameter + "EMPTY_PARAM" + "\")";}
			for (ActivityDefinition b : cd.getBranches(p)) {
				String bname = b.getName();
				if(bname == null || bname.equals("") ){
					bname = "EMPTY_PARAM";
				}
				if (countB < cd.branchesCount(p)) {
					actualParameter = actualParameter + b.getName() + "\"||\"";
				} else {
					actualParameter = actualParameter + b.getName() + "\")";
					param = b.getName();
				}
				countB++;
			}
			formula = formula.replace("\"" + p.getName() + "\"", actualParameter);

		}
		formula = formula.replace("/\\ event==COMPLETE", "");
		formula = formula.replace("/\\ event==complete", "");
		formula = formula.replace("activity==", "");
		formula = formula.replace("_O", "X");
		formula = formula.replace("U_", "U");



		Automaton pAut = new Automaton();
		List<Formula> formulaeParsed = new ArrayList<Formula>();
		try {
			formulaeParsed.add(new DefaultParser(formula).parse());
		} catch (SyntaxParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		TreeFactory<ConjunctionTreeNode, ConjunctionTreeLeaf> treeFactory = DefaultTreeFactory.getInstance();
		ConjunctionFactory<? extends GroupedTreeConjunction> conjunctionFactory = GroupedTreeConjunction
				.getFactory(treeFactory);
		GroupedTreeConjunction conjunction = conjunctionFactory.instance(formulaeParsed);
		pAut = conjunction.getAutomaton().op.reduce();
		ExecutableAutomaton pExecAut = new ExecutableAutomaton(pAut);
		int s = 0;
		for (ExecutionStep step : processTrace) {
			if (!step.equals(ExecutionStep.bottomStep) && logTrace.get(s).equals(ExecutionStep.bottomStep)) {
				if(step.getActivity().equals(ad.getName())){
					added[s] = step.getActivity();
				}
			}
			s++;
		}
		int output = 0;
		for (int d = 0; d < added.length; d++) {
			if (added[d] != null) {
				int t = 0;
				for (ExecutionStep step : processTrace) {
					if (!step.equals(ExecutionStep.bottomStep) && !(d == t)) {
						String label = step.getActivity();
						boolean violated = true;
						PossibleNodes current = null;
						current = pExecAut.currentState();
						for (Transition out : current.output()) {
							if (out.parses(label)) {
								violated = false;
								break;
							}
						}
						if (!violated) {
							pExecAut.next(label);
						} else {
							output ++;
							break;
						}
					}
					t++;
				}
				if(!pExecAut.currentState().isAccepting()){
					output ++;
				}
				pExecAut.ini();
			}			
		}
		return output;
	}




	public static Color getColor(float distance)
	{
		float green=0;
		float blue=0;
		float red=0;
		float intensity=(Math.abs(1-distance)*255F);
		red=(Math.min(intensity,85)*3)/255;
		intensity=Math.max(intensity-85,0);
		green = (Math.min(intensity,85) * 3)/255;
		intensity = Math.max(intensity-85,0);
		blue = (Math.min(intensity,85) * 3)/255;
		return new Color(red,green,blue);
	}

}
