package org.processmining.plugins.DeclareConformance;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.DataConformance.GUI.ActivityMatchCostPanel;
import org.processmining.plugins.DataConformance.GUI.MappingPanel;
import org.processmining.plugins.DataConformance.framework.ActivityMatchCosts;
import org.processmining.plugins.DataConformance.framework.ReplayState;
import org.processmining.plugins.DataConformance.framework.Replayable;
import org.processmining.plugins.DataConformance.framework.VariableMatchCosts;
import org.processmining.plugins.declareminer.visualizing.ActivityDefinition;
import org.processmining.plugins.declareminer.visualizing.AssignmentModel;
import org.processmining.plugins.declareminer.visualizing.AssignmentModelView;
import org.processmining.plugins.declareminer.visualizing.AssignmentViewBroker;
import org.processmining.plugins.declareminer.visualizing.DeclareMap;
import org.processmining.plugins.declareminer.visualizing.XMLBrokerFactory;


public class DeclareConformanceChecking 
{
	
	
	@SuppressWarnings("unchecked")
	@Plugin(name = "Declare Replayer",
			parameterLabels = {"Event log","Declare Model"},
			returnLabels = { "Visualization data" },
			returnTypes = { ResultReplayDeclare.class },
			userAccessible = true,
			help = "Declare Conformance")
			@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "M. de Leoni, F. M. Maggi", email = "m.d.leoni@tue.nl")		
	
	public static ResultReplayDeclare dataConformance(UIPluginContext context, XLog log, DeclareMap model) throws InterruptedException 
	{	
		AssignmentModel hmodel = new AssignmentModel(model.getModel().getLanguage());
		hmodel.setName("new model");


		File file = null;
		try {
			file = File.createTempFile("tempModel", ".xml");
		} catch (IOException e) {
			e.printStackTrace();
		}
		AssignmentViewBroker broker =  XMLBrokerFactory.newAssignmentBroker(file.getAbsolutePath());
		broker.addAssignmentAndView(hmodel, model.getView());
		AssignmentViewBroker mbroker = XMLBrokerFactory.newAssignmentBroker(file.getAbsolutePath());
		AssignmentModel mmodel =mbroker.readAssignment();
		AssignmentModelView mview = new AssignmentModelView(mmodel);
		mbroker.readAssignmentGraphical(mmodel, mview);
		file.delete();
			
		
		XLogInfo summary = XLogInfoFactory.createLogInfo(log);
		XEventClasses eventClasses=summary.getEventClasses();
		HashSet<ReplayableActivityDefinition> activitySet=new HashSet<ReplayableActivityDefinition>();
		for(ActivityDefinition activity: model.getModel().getActivityDefinitions())
			activitySet.add(new ReplayableActivityDefinition(activity.getName()));
		
		//Activity Mapping
		MappingPanel<ReplayableActivityDefinition,XEventClass> mapActivityPanel=
			new MappingPanel<ReplayableActivityDefinition, XEventClass>(activitySet,eventClasses.getClasses());
		InteractionResult result = context.showConfiguration("Setup Activity Mapping", mapActivityPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		Map<ReplayableActivityDefinition,XEventClass> activityMapping=mapActivityPanel.getMapping(false);
		
		
		//Activity Costs
		ActivityMatchCostPanel<XEventClass> activityPanel=new ActivityMatchCostPanel<XEventClass>(activitySet,activityMapping,"Prune Search Space");		
		result = context.showConfiguration("Activity Mapping Cost", activityPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}	
		ActivityMatchCosts<XEventClass> activityCost=activityPanel.getCosts();
		
		VariableMatchCosts variableCost=VariableMatchCosts.NOCOST;
		Map<String,Float> maxDistance=new HashMap<String,Float>();
		
		Progress progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size());
		progBar.setValue(0);
		
		long beforeTime=System.currentTimeMillis();
		
				
		Replayable replayableModel=new ReplayableDeclare(model);
		
		Collection<ReplayState> list=
			MultiThreadDeclareConformanceChecking.perform(replayableModel, activityMapping, new HashMap<String, String>(), 
					eventClasses, log, activityCost, variableCost, maxDistance, context,activityPanel.specialOptionOn());
				
		long afterTime=System.currentTimeMillis();
		
		context.log("Computation time:"+ (afterTime-beforeTime));

		long visitedNode = 0;
		long sizeTree = 0;
		for(ReplayState s : list)
		{
			visitedNode+=s.visitedNodes;
			sizeTree+=s.treeSize;
		}
		context.log("Average number visited nodes: "+(visitedNode/list.size()));
		context.log("Average number nodes of the tree: "+(sizeTree/list.size()));
		
		ResultReplayDeclare retValue=new ResultReplayDeclare(activityMapping,list,activityCost,variableCost,model,log);
		return retValue;
	}
}