package org.processmining.plugins.DeclareConformance;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.plugins.DataConformance.Alignment;
import org.processmining.plugins.DataConformance.visualization.DataAwareConformanceView;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;
import org.processmining.plugins.DataConformance.visualization.ProMPropertiesPanelWithComp;

import com.fluxicon.slickerbox.factory.SlickerFactory;

@Plugin(name = "001 Data-aware View Projection to Log", returnLabels = { "Visualized Log-Model Alignment Projection to Log" }, 
		returnTypes = { JComponent.class }, parameterLabels = { "Matching Instances" }, userAccessible = true)
		@Visualizer
		public class ReplayResultDeclareVis {


	@SuppressWarnings("unchecked")
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, ResultReplayDeclare logReplayResult) {
		SlickerFactory fact=SlickerFactory.instance();
		final ProMPropertiesPanelWithComp logAlignmentPanel = new ProMPropertiesPanelWithComp("Log-Model Alignments. Average Fitness "+
				logReplayResult.meanFitness);
		Collection<Alignment> alignments=logReplayResult.labelStepArray;
		DetailListener first=null;
		ProMSplitPane splitPanel = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);		
		for(Alignment align : alignments) {
			List<String> aLabelList = new ArrayList<String>();
			for(String aLabel : (List<String>) align.getStepLabels())
			{
				aLabel=aLabel.replaceAll("<html>", "").replaceAll("</html>", "").replaceAll("<center>", "").replaceAll("</center>", "").replaceAll("null", "");
				aLabelList.add(aLabel);
			}
			DataAwareConformanceView view=			
				//new DataAwareConformanceView(align.getFitness()+"", align.getStepLabels(),align.getStepTypes());
				new DataAwareConformanceView(align.getFitness()+"", aLabelList,align.getStepTypes());

			JPanel aux=fact.createRoundedPanel();
			JLabel traceLbl=fact.createLabel("Trace "+align.getTraceName());
			traceLbl.setFont(traceLbl.getFont().deriveFont(traceLbl.getFont().getSize()*2F));
			aux.setLayout(new FlowLayout(FlowLayout.CENTER,10,30));
			aux.add(traceLbl);
			JButton detailBtn=fact.createButton("Detail");
			aux.add(detailBtn);
			DetailListener listener=new DetailListener(splitPanel,align,logReplayResult.getDeclareModel());
			if (first==null)
				first=listener;
			detailBtn.addActionListener(listener);
			logAlignmentPanel.addProperty(aux, view);
		}
		ProMSplitPane nordSplitPanel=new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		nordSplitPanel.setLeftComponent(logAlignmentPanel);
		nordSplitPanel.setResizeWeight(1.0);
		nordSplitPanel.setOneTouchExpandable(true);
		nordSplitPanel.setRightComponent(createLegendPanel());
		splitPanel.setBorder(BorderFactory.createEmptyBorder());
		splitPanel.setOneTouchExpandable(true);
		splitPanel.setTopComponent(nordSplitPanel);
		if (first!=null)
			first.showIt();
		//setResizeWeight(1.0);
		splitPanel.setResizeWeight(1.0);
		return splitPanel;	

	}

	protected Component createLegendPanel() {
		SlickerFactory factory = SlickerFactory.instance();

		JPanel legendPanel = new JPanel();
		legendPanel.setBorder(BorderFactory.createEmptyBorder());
		legendPanel.setBackground(new Color(30, 30, 30));
		TableLayout layout = new TableLayout(new double[][] { { 0.10, TableLayout.FILL }, {} });
		legendPanel.setLayout(layout);

		layout.insertRow(0, 0.2);

		int row = 1;

		layout.insertRow(row, TableLayout.PREFERRED);
		JLabel legend = factory.createLabel("LEGEND");
		legend.setForeground(Color.WHITE);
		legendPanel.add(legend, "0,1,1,1,c, c");
		row++;

		layout.insertRow(row, 0.2);

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel greenPanel = new JPanel();
		greenPanel.setBackground(DataAwareConformanceView.getColor(DataAwareStepTypes.LMGOOD));
		legendPanel.add(greenPanel, "0," + row + ",r, c");
		JLabel syncLbl = factory.createLabel("- Perfect Alignment Step (Move log and model)");
		syncLbl.setForeground(Color.WHITE);
		legendPanel.add(syncLbl, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel purplePanel = new JPanel();
		purplePanel.setBackground(DataAwareConformanceView.getColor(DataAwareStepTypes.MREAL));
		legendPanel.add(purplePanel, "0," + row + ",r, c");
		JLabel moveRealLbl = factory.createLabel("- Missing Event (Move model only)");
		moveRealLbl.setForeground(Color.WHITE);
		legendPanel.add(moveRealLbl, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel yellowPanel = new JPanel();
		yellowPanel.setBackground(DataAwareConformanceView.getColor(DataAwareStepTypes.L));
		legendPanel.add(yellowPanel, "0," + row + ",r, c");
		JLabel moveLogLbl = factory.createLabel("- Wrong Event (Move log only)");
		moveLogLbl.setForeground(Color.WHITE);
		legendPanel.add(moveLogLbl, "1," + row++ + ",l, c");


		layout.insertRow(row, 0.2);

		return legendPanel;
	}
}
