package org.processmining.plugins.DeclareConformance;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.processmining.ltl2automaton.plugins.automaton.Automaton;
import org.processmining.ltl2automaton.plugins.automaton.Transition;
import org.processmining.ltl2automaton.plugins.formula.DefaultParser;
import org.processmining.ltl2automaton.plugins.formula.Formula;
import org.processmining.ltl2automaton.plugins.formula.conjunction.ConjunctionFactory;
import org.processmining.ltl2automaton.plugins.formula.conjunction.ConjunctionTreeLeaf;
import org.processmining.ltl2automaton.plugins.formula.conjunction.ConjunctionTreeNode;
import org.processmining.ltl2automaton.plugins.formula.conjunction.DefaultTreeFactory;
import org.processmining.ltl2automaton.plugins.formula.conjunction.GroupedTreeConjunction;
import org.processmining.ltl2automaton.plugins.formula.conjunction.TreeFactory;
import org.processmining.ltl2automaton.plugins.ltl.SyntaxParserException;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.declareminer.ExecutableAutomaton;
import org.processmining.plugins.declareminer.PossibleNodes;
import org.processmining.plugins.declareminer.visualizing.ActivityDefinition;
import org.processmining.plugins.declareminer.visualizing.AssignmentModel;
import org.processmining.plugins.declareminer.visualizing.ConstraintDefinition;
import org.processmining.plugins.declareminer.visualizing.DeclareMap;
import org.processmining.plugins.declareminer.visualizing.Parameter;



public class ViolationIdentifier {

	Vector[] violations;
	Vector[] single;
	Vector[] contributions;
	//HashMap activations;
	DeclareMap declareModel;
	HashMap successors;
	Vector alreadyVisited;
	String[]  skipped;
	String[]  added;
	List<ExecutionStep> logTrace;
	List<ExecutionStep> processTrace;


	public ViolationIdentifier(List<ExecutionStep> logTrace, List<ExecutionStep> processTrace, DeclareMap declareModel) {
		HashMap violInstances = new HashMap();
		this.logTrace = logTrace;
		this.processTrace = processTrace;
		//activations = new HashMap();
		successors = new HashMap();
		this.declareModel = declareModel;
		alreadyVisited = new Vector();
		skipped  = new String[logTrace.size()];
		added  = new String[logTrace.size()];
		violations = new Vector[logTrace.size()];
		single = new Vector[logTrace.size()];
		contributions = new Vector[logTrace.size()];
		AssignmentModel amodel = declareModel.getModel();
		String[] constraints = new String[amodel.constraintDefinitionsCount()];
		String[] forms = new String[amodel.constraintDefinitionsCount()];
		Vector[] vacuityDetection = new Vector[amodel.constraintDefinitionsCount()];
		int i = 0;
		String formula = "(";
		for (ConstraintDefinition cd : amodel.getConstraintDefinitions()) {
			constraints[i] = cd.getName();
			int count = 1;
			constraints[i] = constraints[i] + "(";
			for (Parameter p : cd.getParameters()) {
				if (count < cd.parameterCount()) {
					int countB = 1;
					constraints[i] = constraints[i] + "[";
					if(cd.getBranches(p).size()==0){constraints[i] = constraints[i] + "EMPTY_PARAM" + "\"]";}
					for (ActivityDefinition b : cd.getBranches(p)) {
						String bname = b.getName();
						if(bname == null || bname == null || bname.equals("")  ){
							bname = "EMPTY_PARAM";
						}
						if (countB < cd.branchesCount(p)) {
							constraints[i] = constraints[i] + bname + ",";
						} else {
							constraints[i] = constraints[i] + bname + "],";
						}
						countB++;
					}
				} else {
					int countB = 1;
					constraints[i] = constraints[i] + "[";
					if(cd.getBranches(p).size()==0){constraints[i] = constraints[i] + "EMPTY_PARAM" + "\"]";}
					for (ActivityDefinition b : cd.getBranches(p)) {
						String bname = b.getName();
						if(bname == null || bname.equals("") ){
							bname = "EMPTY_PARAM";
						}
						if (countB < cd.branchesCount(p)) {
							constraints[i] = constraints[i] +bname + ",";
						} else {
							constraints[i] = constraints[i] + bname + "])";
						}
						countB++;
					}
				}
				count++;
			}
			String currentFA = cd.getText();
			String currentFB = null;
			
			if (cd.getName().equals("absence")){
				currentFA = "!( <> ( \"A\" ) )";
			}
			if (cd.getName().equals("absence2")){
				currentFA = "! ( <> ( ( \"A\" /\\ X(<>(\"A\")) ) ) )";
			}
			if (cd.getName().equals("absence3")){
				currentFA = "! ( <>( ( <> (\"A\") /\\ X( ( <> (\"A\") /\\  X ( <> ( \"A\" ) )) ) ) ) )";
			}
			if (cd.getName().equals("alternate precedence")){
				currentFA = "( ( (! ( \"B\" ) U \"A\") \\/ ([](!(\"B\"))) ) /\\ [] ( ( \"B\" -> X( ( ( ! ( \"B\" ) U \"A\" )\\/([](!(\"B\"))) )) ) ) ) /\\ ! (\"B\" )";
			}
			if (cd.getName().equals("alternate response")){
				currentFA = "( []( ( \"A\" -> X(( (! ( \"A\" )) U \"B\" ) )) ) )";
			}
			if (cd.getName().equals("alternate succession")) {
				currentFA = "( []( ( \"A\" -> X(( ! ( \"A\" ) U \"B\" ) )) ) )";// /\\ ( ( (! ( \"B\" ) U \"A\") \\/ ([](!(\"B\"))) ) /\\ [] ( ( \"B\" -> X( ( ( ! ( \"B\" ) U \"A\" )\\/([](!(\"B\"))) )) ) ) )" ;
				currentFB = " ( ( (! ( \"B\" ) U \"A\") \\/ ([](!(\"B\"))) ) /\\ [] ( ( \"B\" -> X( ( ( ! ( \"B\" ) U \"A\" )\\/([](!(\"B\"))) )) ) ) )  /\\ ! (\"B\" )";
			}
			if (cd.getName().equals("chain precedence")){
				currentFA = "[]( ( X( \"B\" ) -> \"A\") )/\\ ! (\"B\" )";
			}
			if (cd.getName().equals("chain response")){
				currentFA = "[] ( ( \"A\" -> X( \"B\" ) ) )";
			}
			if (cd.getName().equals("chain succession")) {
				currentFA = "[]( ( \"A\" -> X( \"B\" ) ) )";
				currentFB = "[]( ( X( \"B\" ) ->  \"A\") ) /\\ ! (\"B\" )";
			}
			if (cd.getName().contains("of")){
				currentFA = "(  <> ( \"A\" ) \\/ <>( \"B\" )  )";
			}
			if (cd.getName().equals("co-existence")) {
				currentFA = "( ( <>( \"A\" ) -> (<>( \"B\" ) )) )";
				currentFB = "( ( <>( \"B\" ) -> (<>( \"A\" ) )) )";
			}
			if (cd.getName().equals("exactly1")){
				currentFA = "(  <> (\"A\") /\\ ! ( <> ( ( \"A\" /\\ X(<>(\"A\")) ) ) ) )";
			}
			if (cd.getName().equals("exactly2")){
				currentFA = "( <> (\"A\" /\\ (\"A\" -> (X(<>(\"A\"))))) /\\  ! ( <>( \"A\" /\\ (\"A\" -> X( <>( \"A\" /\\ (\"A\" -> X ( <> ( \"A\" ) ))) ) ) ) ) )";
			}
			if (cd.getName().equals("exclusive choice")){
				currentFA = "(  ( <>( \"A\" ) \\/ <>( \"B\" )  )  /\\ !( (  <>( \"A\" ) /\\ <>( \"B\" ) ) ) )";
			}
			if (cd.getName().equals("existence")){
				currentFA = "( <> ( \"A\" ) )";
			}
			if (cd.getName().equals("existence2")){
				currentFA = "<> ( ( \"A\" /\\ X(<>(\"A\")) ) )";
			}
			if (cd.getName().equals("existence3")){
				currentFA = "<>( \"A\" /\\ X(  <>( \"A\" /\\ X( <> \"A\" )) ))";
			}
			if (cd.getName().equals("strong init")){
				currentFA = "( \"A\" )";
			}
			if (cd.getName().equals("init")){
				currentFA = "( \"A\" )";
			}
			if (cd.getName().equals("not chain succession")){
				currentFA = "[]( ( \"A\" -> !(X( \"B\" ) ) ))";
				currentFB = "[]( ( X( \"B\" ) -> ! (\"A\" ) ) )";
			}
			if (cd.getName().equals("not co-existence")) {
				currentFA = "( ( <>( \"A\" ) -> (!<>( \"B\" ) )) )";
				currentFB = "( ( <>( \"B\" ) -> (!<>( \"A\" ) )) )";
			}
			if (cd.getName().equals("not succession")) {
				currentFA = "[]( ( \"A\" -> !(<>( \"B\" ) ) ))";
				currentFB = "( ! (\"A\" ) U \"B\" ) \\/ ([](!(\"B\")))";
			}
			if (cd.getName().equals("precedence")){
				currentFA = "( ! (\"B\" ) U \"A\" ) \\/ ([](!(\"B\"))) /\\ ! (\"B\" )";
			}
			if (cd.getName().equals("responded existence")){
				currentFA = "(( ( <>( \"A\" ) -> (<>( \"B\" ) )) ))";
			}
			if (cd.getName().equals("response")){
				currentFA = "( []( ( \"A\" -> <>( \"B\" ) ) ))";
			}
			if (cd.getName().equals("succession")) {
				currentFA = "( []( ( \"A\" -> <>( \"B\" ) ) ))";// /\\ (( ! (\"B\" ) U \"A\" ) \\/ ([](!(\"B\"))) /\\ ! (\"B\" ))"; 
				currentFB = "( ! (\"B\" ) U \"A\" ) \\/ ([](!(\"B\"))) /\\ ! (\"B\" )";
			}
			
			
			
			
			
			
			
		
			
			Vector params = new Vector();
			int paramCount = 0;
			for (Parameter p : cd.getParameters()) {
				int countB = 1;
				String param = null;
				String actualParameter = "(\"";
				if(cd.getBranches(p).size()==0){actualParameter = actualParameter + "EMPTY_PARAM" + "\")"; param = "EMPTY_PARAM";}
				for (ActivityDefinition b : cd.getBranches(p)) {
					String bname = b.getName();
					if(bname == null || bname.equals("") ){
						bname = "EMPTY_PARAM";
					}
					if (countB < cd.branchesCount(p)) {
						actualParameter = actualParameter + bname + "\"||\"";
					} else {
						actualParameter = actualParameter + bname + "\")";
						param = bname;
					}
					countB++;
				}
				currentFA = currentFA.replace("\"" + p.getName() + "\"", actualParameter);
				if(currentFB!= null){
					currentFB = currentFB.replace("\"" + p.getName() + "\"", actualParameter);
				}
				if(cd.getName().equals("co-existence")){
					params.add(param);
				}else if(cd.getName().equals("responded existence")){
					params.add(param);
				}else if(cd.getName().equals("precedence") && paramCount ==1){
					params.add(param);
				}else if(cd.getName().equals("response") && paramCount ==0){
					params.add(param);
				}else if(cd.getName().equals("succession")){
					params.add(param);
				}else if(cd.getName().equals("alternate precedence") && paramCount ==1){
					params.add(param);
				}else if(cd.getName().equals("alternate response") && paramCount ==0){
					params.add(param);
				}else if(cd.getName().equals("alternate succession")){
					params.add(param);
				}else if(cd.getName().equals("chain precedence") && paramCount ==1){
					params.add(param);
				}else if(cd.getName().equals("chain response") && paramCount ==0){
					params.add(param);
				}else if(cd.getName().equals("chain succession")){
					params.add(param);
				}else if(cd.getName().equals("not co-existence")){
					params.add(param);
				}else if(cd.getName().equals("not succession")){
					params.add(param);
				}else if(cd.getName().equals("not chain succession")){
					params.add(param);
				}else if(cd.getName().equals("not response") && paramCount ==0){
					params.add(param);
				}else if(cd.getName().equals("not chain response") && paramCount ==0){
					params.add(param);
				}else if(cd.getName().equals("not precedence") && paramCount ==1){
					params.add(param);
				}else if(cd.getName().equals("not chain precedence") && paramCount ==1){
					params.add(param);
				}else{
					params.add(param);
				}
				paramCount ++;
			}
			currentFA = currentFA.replace("/\\ event==COMPLETE", "");
			currentFA = currentFA.replace("/\\ event==complete", "");
			currentFA = currentFA.replace("activity==", "");
			currentFA = currentFA.replace("_O", "X");
			currentFA = currentFA.replace("U_", "U");

			if(currentFB!= null){
				currentFB = currentFB.replace("/\\ event==COMPLETE", "");
				currentFB = currentFB.replace("/\\ event==complete", "");
				currentFB = currentFB.replace("activity==", "");
				currentFB = currentFB.replace("_O", "X");
				currentFB = currentFB.replace("U_", "U");
			}
			vacuityDetection[i]=params;
			if(currentFB!= null){
				forms[i] = currentFA +";"+currentFB;
			}else{
				forms[i] = currentFA;
			}

			i++;
		}
		Automaton[] pAut = new Automaton[forms.length];
		ExecutableAutomaton[] pExecAutA = new ExecutableAutomaton[forms.length];
		ExecutableAutomaton[] pExecAutB = new ExecutableAutomaton[forms.length];
		for (int j = 0; j < forms.length; j++) {
			String[] parts = forms[j].split(";");
			List<Formula> formulaeParsed = new ArrayList<Formula>();
			try {
				formulaeParsed.add(new DefaultParser(parts[0]).parse());
			} catch (SyntaxParserException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			TreeFactory<ConjunctionTreeNode, ConjunctionTreeLeaf> treeFactory = DefaultTreeFactory.getInstance();
			ConjunctionFactory<? extends GroupedTreeConjunction> conjunctionFactory = GroupedTreeConjunction
					.getFactory(treeFactory);
			GroupedTreeConjunction conjunction = conjunctionFactory.instance(formulaeParsed);
			pAut[j] = conjunction.getAutomaton().op.reduce();
			pExecAutA[j] = new ExecutableAutomaton(pAut[j]);
			pExecAutB[j] = null;
			if(parts.length>1){
				formulaeParsed = new ArrayList<Formula>();
				try {
					formulaeParsed.add(new DefaultParser(parts[1]).parse());
				} catch (SyntaxParserException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				treeFactory = DefaultTreeFactory.getInstance();
				conjunctionFactory = GroupedTreeConjunction
						.getFactory(treeFactory);
				conjunction = conjunctionFactory.instance(formulaeParsed);
				pAut[j] = conjunction.getAutomaton().op.reduce();
				pExecAutB[j] = new ExecutableAutomaton(pAut[j]);
			}
		}
		for (int j = 0; j < pAut.length; j++) {
			pExecAutA[j].ini();
			if(pExecAutB[j]!=null){
				pExecAutB[j].ini();
			}
		}

		int s = 0;
		Vector violatedConstraints =null;
		for(ExecutionStep step : processTrace){
			violations[s] =new Vector();
			single[s] =new Vector();
			contributions[s] =new Vector();
			//skipped[s]=new Vector();
			//added[s]=new Vector();
			if(step.equals(ExecutionStep.bottomStep) && !logTrace.get(s).equals(ExecutionStep.bottomStep)){
				skipped[s] = logTrace.get(s).getActivity(); 
			} 
			if(!step.equals(ExecutionStep.bottomStep) && logTrace.get(s).equals(ExecutionStep.bottomStep)){
				added[s] = step.getActivity(); 
			} 
			s++;
		}




		for(int d=0; d<skipped.length; d++) {
			if(skipped[d] != null){
				Vector constrSucc = new Vector();
				violatedConstraints = new Vector();

				for (int j = 0; j < pAut.length; j++) {
					pExecAutA[j].ini();
					if(pExecAutB[j]!=null){
						pExecAutB[j].ini();
					}
					int t = 0;
					int stepNumber = -1;
					PossibleNodes currentA = null;
					PossibleNodes currentB = null;
					currentA = pExecAutA[j].currentState();
					if(pExecAutB[j]!=null){
						currentB = pExecAutB[j].currentState();
					}
					for(ExecutionStep step : processTrace){
						stepNumber ++;
						currentA = pExecAutA[j].currentState();
						if(pExecAutB[j]!=null){
							currentB = pExecAutB[j].currentState();
						}
						if(!step.equals(ExecutionStep.bottomStep)||(d==t)){

							String label = null;
							if(!step.equals(ExecutionStep.bottomStep)){	
								label = step.getActivity();
							}
							if(d==t){
								label = logTrace.get(t).getActivity();
							}
							boolean violated = true;

							for (Transition out : currentA.output()) {
								if (out.parses(label)) {
									violated = false;
									break;
								}
							}
							if(!violated && pExecAutB[j]!=null){
								violated = true;
								for (Transition out : currentB.output()) {
									if (out.parses(label)) {
										violated = false;
										break;
									}
								}
							}

							if(!violated){
								pExecAutA[j].next(label);
								if(pExecAutB[j]!=null){
									pExecAutB[j].next(label);
								}
							}else{
								if(!contributions[d].contains(constraints[j])){
									contributions[d].add(constraints[j]);
								}
								break;
							}
						}
						t++;
					}
					currentA = pExecAutA[j].currentState();
					if(pExecAutB[j]!=null){
						currentB = pExecAutB[j].currentState();
					}
					if(!currentA.isAccepting()){
						if(!contributions[d].contains(constraints[j])){
							contributions[d].add(constraints[j]);
						}
					}
					if(pExecAutB[j]!=null){
						if(!currentB.isAccepting()){
							if(!contributions[d].contains(constraints[j])){
								contributions[d].add(constraints[j]);
							}
						}
					}
				}
			}
		}
		//act = null;
		violInstances = new HashMap();
		for(int d=0; d<added.length; d++) {
			if(added[d]!= null){
				for (int j = 0; j < pAut.length; j++) {
					pExecAutA[j].ini();
					if(pExecAutB[j]!=null){
						pExecAutB[j].ini();
					}
					int stepNumber = -1;
					int t = 0;
					PossibleNodes currentA = null;
					PossibleNodes currentB = null;
					currentA = pExecAutA[j].currentState();
					if(pExecAutB[j]!=null){
						currentB = pExecAutB[j].currentState();
					}
					for(ExecutionStep step : processTrace){
						stepNumber++;
						currentA = pExecAutA[j].currentState();
						if(pExecAutB[j]!=null){
							currentB = pExecAutB[j].currentState();
						}
					
						if(!step.equals(ExecutionStep.bottomStep)&&!(d==t)){	

							String label = step.getActivity();
							boolean violated = true;
							for (Transition out : currentA.output()) {
								if (out.parses(label)) {
									violated = false;
									break;
								}
							}
							if(!violated && pExecAutB[j]!=null){
								violated = true;
								for (Transition out : currentB.output()) {
									if (out.parses(label)) {
										violated = false;
										break;
									}
								}
							}

							if(!violated){
								pExecAutA[j].next(label);
								if(pExecAutB[j]!=null){
									pExecAutB[j].next(label);
								}
							}else{
								if(!contributions[d].contains(constraints[j])){
									contributions[d].add(constraints[j]);
								}
								break;
							}
						}
						t++;
					}
					currentA = pExecAutA[j].currentState();
					if(pExecAutB[j]!=null){
						currentB = pExecAutB[j].currentState();
					}
					if(!currentA.isAccepting()){
						if(!contributions[d].contains(constraints[j])){
							contributions[d].add(constraints[j]);
						}
					}
					if(pExecAutB[j]!=null){
						if(!currentB.isAccepting()){
							if(!contributions[d].contains(constraints[j])){
								contributions[d].add(constraints[j]);
							}
						}
					}
				}
			}
		}
	}

	public int getNumberConstraintActivations() 
	{
		int result = 0;	
		AssignmentModel amodel = declareModel.getModel();
		for (ConstraintDefinition cd : amodel.getConstraintDefinitions()) {
			result = result + this.getNumberConstraintActivations(cd);
		}
		return result;
	}


	public int getNumberConstraintActivations(ConstraintDefinition cd) 
	{
		String constraint = cd.getName();
		int count = 1;
		constraint = constraint + "(";
		for (Parameter p : cd.getParameters()) {
			if (count < cd.parameterCount()) {
				int countB = 1;
				constraint = constraint + "[";
				if(cd.getBranches(p).size()==0){constraint = constraint + "EMPTY_PARAM" + "\"]";}
				for (ActivityDefinition b : cd.getBranches(p)) {
					String bname = b.getName();
					if(bname == null || bname.equals("") ){
						bname = "EMPTY_PARAM";
					}
					if (countB < cd.branchesCount(p)) {
						constraint = constraint + bname + ",";
					} else {
						constraint = constraint + bname + "],";
					}
					countB++;
				}
			} else {
				int countB = 1;
				constraint = constraint + "[";
				if(cd.getBranches(p).size()==0){constraint = constraint + "EMPTY_PARAM" + "\"]";}
				for (ActivityDefinition b : cd.getBranches(p)) {
					String bname = b.getName();
					if(bname == null || bname.equals("") ){
						bname = "EMPTY_PARAM";
					}
					if (countB < cd.branchesCount(p)) {
						constraint = constraint + bname + ",";
					} else {
						constraint = constraint + bname + "])";
					}
					countB++;
				}
			}
			count++;
		}
		//		return ((Integer)activations.get(constraint)).intValue();
		return 0;
	}


	public int getNumberConstraintViolations()
	{
		Vector alreadyCounted = new Vector();
		int result = 0;
		for(int i=0; i < violations.length; i++){
			for(int j =0; j < violations[i].size(); j++){
				if(!alreadyCounted.contains(violations[i].get(j))){
					alreadyCounted.add(violations[i].get(j));
					result ++;
				}
			}

		}
		return result;
	}

	/*	public String[] getViolationsContributedToSolve(int step) {
		Vector result = violations[step];
		if(contributions[step].size()==0){
			for(int i = 0; i<result.size(); i++){
				boolean singleStepSolving = true; 
				String violConstr = (String)result.get(i);
				for(int j = 0; j<violations.length; j++){
					for(int k = 0; k<violations[j].size(); k++){
						if(violations[j].get(k).equals(violConstr)&&j!=step){
							singleStepSolving = false;
							int position = (new Integer(violConstr.split(";")[1].split("position ")[1])).intValue()+1;
							if(!contributions[step].contains(violConstr.split(";")[0].substring(14))){
							contributions[step].add(violConstr.split(";")[0].substring(14));}
							break;
						}
					}
					if(!singleStepSolving){
						break;
					}
				}
			}
		}
		return (String[]) (contributions[step]).toArray(new String[0]);
	}*/

	/*public String[] getViolationsSolved(int step) {
		Vector result = violations[step]; 
		if(single[step].size()==0){
			for(int i = 0; i<result.size(); i++){
				boolean singleStepSolving = true; 
				String violConstr = (String)result.get(i);
				for(int j = 0; j<violations.length; j++){
					for(int k = 0; k<violations[j].size(); k++){
						if(violations[j].get(k).equals(violConstr)&&j!=step){
							singleStepSolving = false;
							break;
						}
					}
					if(!singleStepSolving){
						break;
					}
				}
				if(singleStepSolving){
					int position = (new Integer(violConstr.split(";")[1].split("position ")[1])).intValue()+1;
					if(!single[step].contains(violConstr.split(";")[0].substring(14))){
					single[step].add(violConstr.split(";")[0].substring(14));}
				}
			}
		}
		return (String[]) (single[step]).toArray(new String[0]);
	}*/



	public String[] getViolationsContributedToSolve(int step) {
	//	return (String[]) (contributions[step]).toArray(new String[0]);
		return new String[0];
	}

	public String[] getViolationsSolved(int step) {
		return (String[]) (contributions[step]).toArray(new String[0]);	
	}


	public void visit(String node, String arc, int d){
		alreadyVisited.add(node);
		if(successors.containsKey(node)){
			Vector succs = (Vector)successors.get(node);
			for(int i = 0; i < succs.size(); i++){
				if(!alreadyVisited.contains(((String)succs.get(i)).split("constr;")[0])){
					visit(((String)succs.get(i)).split("constr;")[0],((String)succs.get(i)).split("constr;")[1],d);
				}
			}
		}else{
			violations[d].add("1 instance of "+(arc+";"+node));
		}
		return;
	}

}
