package org.processmining.plugins.DeclareDataAwareComplianceChecker.automata;

import java.util.ArrayList;
import java.util.Collection;

public class State {

	public final Collection<Transition> output;
	public boolean accepting;
	public int id = 0;
	public boolean satisfied;
	
	public State() {
		this(0);
	}

	public State(final int id) {
		super();
		setId(id);
		setAccepting(false);

		output = new ArrayList<Transition>();
	}

	public synchronized int getId() {
		return id;
	}

	public Iterable<Transition> getOutput() {
		return output;
	}

	public int getOutputSize() {
		return output.size();
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + id;
		return hash;
	}

	public boolean isAccepting() {
		return accepting;
	}
	
	public boolean isSatisfied() {
		return satisfied;
	}
	
	public synchronized void removeAllTransitions() {
		output.clear();
	}

	@Override
	public String toString() {
		return "S" + id;
	}

	public synchronized void addOutput(final Transition e) {
		output.add(e);
	}

	public synchronized void removeOutput(final Transition e) {
		output.remove(e);
	}

	public void setAccepting(final boolean acc) {
		accepting = acc;
	}
	
	public void setSatisfied(final boolean sat) {
		satisfied = sat;
	}

	public synchronized void setId(final int id) {
		this.id = id;
	}
}

