package org.processmining.plugins.DeclareDataAwareComplianceChecker.models;

import java.util.Hashtable;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.processmining.models.guards.Expression;
import org.processmining.plugins.DeclareDataAwareComplianceChecker.automata.Automaton;
import org.processmining.plugins.DeclareDataAwareComplianceChecker.automata.ExecutableAutomaton;
import org.processmining.plugins.DeclareDataAwareComplianceChecker.automata.State;

/*
 * Author: M.H.M. Schouten
 * E-mail: m.h.m.schouten0@gmail.com
 * 
 */

public class DataDeclareConstraint extends Automaton{
	
	State initial;
	ExecutableAutomaton exec;

	String activation;
	String target;
	Expression guard;
	
	final String rest = "";
	
	boolean activatorPresent;
	
	// Create an executable automaton for the given automaton
	public void createExecAutomaton(Automaton a){
		exec = new ExecutableAutomaton(a);
	}
	
	public DataDeclareConstraint(String first, String second, Expression exp){
		super();
		activation = first;
		target = second;
		guard = exp;
		activatorPresent = false;
		
		//Create an initial state for the automaton
		initial = new State(0);
		this.addState(initial);
		this.setInitial(initial);
	}
	
	// Determine whether the automaton can make a valid transition
	public boolean nextActivity(XEvent event) {
		
		String actName = XConceptExtension.instance().extractName(event);
		if( !actName.equals(activation) && !actName.equals(target) ){
			actName = rest;
		}
		
		XAttributeMap atts = event.getAttributes();
		Hashtable<String, String> numbers = new Hashtable<String, String>();
		Set keyS = atts.keySet();
		for(Object key : keyS){
			XAttribute value = atts.get(key);
			numbers.put(key.toString(), value.toString());
		}
		
		if(guard == null && actName.equals(activation)){
			actName = activation+"true";
			activatorPresent = true;
		}
		else if( actName.equals(activation)){
			if (guard.isTrue(numbers)){
				actName = activation+"true";
				activatorPresent = true;
			}
			else {
				actName = activation+"false";
			}
		}
		if(exec.next(actName) != null){
			return true;
		}
		else {
			return false;
		}
	}
	
	//Return whether the current state is a final state
	public boolean isInFinalState() {
		if(exec.currentState() != null){
			if (exec.currentState().isAccepting()){
				return true;
			}
			else {
				return false;
			}
		}
		else return false;
	}
	
	public String getActivation(){
		return activation;
	}
	
	// This method resets the automaton back to the initial state by recreating it
	public void reset() {
		Automaton temp = exec.getAutomaton();
		exec = new ExecutableAutomaton(temp);
		activatorPresent = false;
	}
	
	public boolean isActivatorPresent(){
		return activatorPresent;
	}

}
