/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.DeclareDataAwareComplianceChecker.visualization;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Rectangle2D;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import org.deckfour.uitopia.api.model.ResourceType;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryNaiveImpl;
import org.deckfour.xes.model.XAttributable;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.jgraph.graph.DefaultGraphCell;
import org.processmining.contexts.uitopia.UIContext;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.model.ProMPOResource;
import org.processmining.contexts.uitopia.model.ProMTask;
import org.processmining.framework.plugin.GlobalContext;
import org.processmining.framework.providedobjects.ProvidedObjectID;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.plugins.DeclareDataAwareComplianceChecker.parameters.ComplianceCheckerOutput;
import org.processmining.plugins.DeclareMapWithData.models.DataDeclareMap;
import org.processmining.plugins.declare.visualizing.AssignmentModelView;
import org.processmining.plugins.declare.visualizing.ConstraintDefinition;
import org.processmining.plugins.declare.visualizing.DGraph;
import org.processmining.plugins.declareminer.visualizing.Parameter;
import org.processmining.plugins.utils.ProvidedObjectHelper;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.organic.JGraphOrganicLayout;




/**
 * 
 * Author: M.H.M. Schouten
 * E-mail: m.h.m.schouten0@gmail.com
 * 
 * 
 * 
 */
public class DataDeclareModelVisualizer extends JPanel implements MouseListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 215385982744090270L;
	
	protected DGraph graph;
	protected ProMSplitPane mainPanel;
	protected JScrollPane leftPanel;
	protected JPanel rightPanel;
	
	DataDeclareMap decModel;
	JGraphOrganicLayout oc;
	
	String[] displays;
	String[] descriptions;
	
	JPopupMenu m_nodeMenu;
	JMenuItem m_extractfit;
	JMenuItem m_extractnotfit;
	UIPluginContext cont;
	
	private JButton toggleView;
	
	Rectangle clickedBounds;
	
	HashMap<org.processmining.plugins.declareminer.visualizing.ConstraintDefinition, ArrayList<String>> failedTracesMap;
	XLog usedLog;

	@Override
	public void mouseClicked(final MouseEvent arg0) {
		if (graph != null) {
			graph.refresh();
			graph.repaint();
		}
		if (mainPanel != null) {
			mainPanel.repaint();
		}	    
	}

	@Override
	public void mouseEntered(final MouseEvent arg0) {
		// TODO Auto-generated method stub
		if (mainPanel != null) {
			mainPanel.repaint();
		}
	}

	@Override
	public void mouseExited(final MouseEvent arg0) {
		// TODO Auto-generated method stub
		if (mainPanel != null) {
			mainPanel.repaint();
		}
	}

	@Override
	public void mousePressed(final MouseEvent arg0) {
		if (graph != null) {
			graph.refresh();
			graph.repaint();
		}
		if (mainPanel != null) {
			mainPanel.repaint();
		}

	}

	@Override
	public void mouseReleased(final MouseEvent arg0) {
		if (graph != null) {
			graph.refresh();
			graph.repaint();
		}
		if (mainPanel != null) {
			mainPanel.repaint();
		}
	}
	
	HashMap<ConstraintDefinition, org.processmining.plugins.declareminer.visualizing.ConstraintDefinition> constraintMap = new HashMap<ConstraintDefinition, org.processmining.plugins.declareminer.visualizing.ConstraintDefinition>();
	
	public JComponent getJComponent(final UIPluginContext context, final ComplianceCheckerOutput dmap) {
		cont = context;
		decModel = dmap;
		usedLog = dmap.getLog();
		failedTracesMap = dmap.getFailedTracesMap();
		
		SlickerFactory sf = SlickerFactory.instance();
		final org.processmining.plugins.declareminer.visualizing.ConstraintDefinition[] CDarray = dmap.getCDarray();
		
		int j = 0;
		displays = new String[decModel.getModelCh().getConstraintDefinitions().size()];
		descriptions = new String[decModel.getModelCh().getConstraintDefinitions().size()];
		
		ConstraintDefinition[] array1 = new ConstraintDefinition[decModel.getModelCh().getConstraintDefinitions().size()];
		
		int counter = 0;
		for(ConstraintDefinition cd : decModel.getModelCh().getConstraintDefinitions()){
			array1[counter] = cd;
			counter++;
		}
		counter = 0;
		for(org.processmining.plugins.declareminer.visualizing.ConstraintDefinition cd : decModel.getModel().getConstraintDefinitions()){
			constraintMap.put(array1[counter], cd);
			counter++;

		}
		
		for(ConstraintDefinition cd : decModel.getModelCh().getConstraintDefinitions()){
			if(cd.getParameters().size() > 1){	
				displays[j] = CDarray[j].getDisplay();
				descriptions[j] = CDarray[j].getDescription();  
				j++;
			}
		}
		
		j=0;
		for(ConstraintDefinition cd : decModel.getModelCh().getConstraintDefinitions()){
			if(cd.getParameters().size() > 1){
				double ratio = Double.parseDouble(descriptions[j]);
				//int col = (int) Math.round(255* (1 - ratio) );
				//Color cdCol = new Color(col, 0, 0);
				Color cdCol = getColor(1-ratio);
				Color gray = Color.LIGHT_GRAY;
				decModel.getViewCh().setConstraintDefinitionColor(cd, cdCol, cdCol, gray);	
				j++;
			}
		}
	
		graph = decModel.getViewCh().getGraph();

		graph.addMouseListener(this);
		//scroll.addMouseListener(this);
		graph.setBackground(Color.LIGHT_GRAY);
		graph.doLayout();
		graph.refresh();
		graph.repaint();
		
		leftPanel = new JScrollPane(graph);
		mainPanel = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		rightPanel = sf.createRoundedPanel();
		
		int gridy = 3;

		setLayout(new GridBagLayout());
		
		toggleView = sf.createButton("Toggle Label View");
		toggleView.addMouseListener(new MouseAdapter() {

				public void mouseReleased(MouseEvent e) {
				}
				
				public void mouseExited(MouseEvent e) {
				}

				public void mouseEntered(MouseEvent e) {
				}

				public void mouseClicked(MouseEvent e) {
				}
				
				public void mousePressed(MouseEvent e) {
					int i = 0;
					
					for(ConstraintDefinition cd : decModel.getModelCh().getConstraintDefinitions()){
						if(cd.getParameters().size() > 1 ){
							
							//String newLabel = newLabels[i]; //description
							//String currentLabel = currentLabels[i]; //display
							//String newLabel = CDarray[i].getDescription(); //description
							//String currentLabel = CDarray[i].getDisplay(); //display
							String currentLabel = cd.getDisplay(); //display
							
							if (currentLabel.equals(displays[i]) ){
								cd.setDisplay(descriptions[i]);
							}
							if (currentLabel.equals(descriptions[i]) ){
								cd.setDisplay(displays[i]);
							}
							decModel.getViewCh().updateConstraintDefinition(cd);
							
							i++;
						}
					}

					AssignmentModelView viewCh = new AssignmentModelView(decModel.getModelCh());
					
					i = 0;
					for(ConstraintDefinition cd : decModel.getModelCh().getConstraintDefinitions()){
						if(cd.getParameters().size() > 1 ){
							double ratio = Double.parseDouble(descriptions[i]);
							Color cdCol = getColor(1-ratio);
							//int col = (int) Math.round(255* (1 - ratio) );
							//Color cdCol = new Color(col, 0, 0);
							Color gray = Color.LIGHT_GRAY;
							viewCh.setConstraintDefinitionColor(cd, cdCol , cdCol, gray);	
							i++;
						}
					}
					
					decModel = new DataDeclareMap(decModel.getModel(),decModel.getView(),null, decModel.getModelCh(), viewCh, null);
					decModel.setCDarray(dmap.getCDarray());
					decModel.setCDNamearray(dmap.getCDNamearray());
					decModel.setGuards(dmap.getGuards());
					decModel.setVariables(dmap.getVariables());
//					
					graph = decModel.getViewCh().getGraph();
					graph.setBackground(Color.LIGHT_GRAY);
					
					
					final JGraphFacade jgf = new JGraphFacade(graph);
					oc.run(jgf);
					final Map nestedMap = jgf.createNestedMap(true, true); 
					graph.getGraphLayoutCache().edit(nestedMap); 
					createPopupMenu();
					graph.doLayout();
					graph.refresh();
					graph.repaint();
					
					leftPanel = new JScrollPane(graph);
					leftPanel.revalidate();
					leftPanel.repaint();

					mainPanel.setLeftComponent(leftPanel);
					mainPanel.repaint();
					revalidate();
					repaint();
				}
				
				

		});
		
		rightPanel.setLayout(new TableLayout(new double[][] { { 20, TableLayoutConstants.FILL}, {50,  20, 20}}));
		
		rightPanel.add(toggleView, "1,1");
		//rightPanel.add(showSuccesRatioBox, "1,2");
		
		mainPanel.setLeftComponent(leftPanel);
		mainPanel.setRightComponent(rightPanel);
		mainPanel.setDividerLocation(1700);
		mainPanel.setOneTouchExpandable(true);
		//ProMScrollPane scrollPanel = new ProMScrollPane(histogram);
		

		oc = new JGraphOrganicLayout();
		oc.setDeterministic(true);
		oc.setOptimizeBorderLine(true);
		oc.setOptimizeEdgeCrossing(true);
		oc.setOptimizeEdgeDistance(true);
		oc.setOptimizeEdgeLength(true);
		oc.setOptimizeNodeDistribution(true);
		oc.setEdgeCrossingCostFactor(999999999);
		oc.setEdgeDistanceCostFactor(999999999);
		oc.setFineTuning(true);
		oc.setEdgeLengthCostFactor(9999);
		oc.setEdgeLengthCostFactor(99);
		oc.setNodeDistributionCostFactor(999999999);
		oc.setBorderLineCostFactor(999);
		oc.setRadiusScaleFactor(0.9);
		final JGraphFacade jgf = new JGraphFacade(graph);
		oc.run(jgf);
		final Map nestedMap = jgf.createNestedMap(true, true); 
		graph.getGraphLayoutCache().edit(nestedMap); 
		createPopupMenu();
	    
	    //mainPanel.add(m_nodeMenu);  
		return mainPanel;
	}
	

	//Pop up menu for extracting logs 
	private void createPopupMenu(){
		m_nodeMenu = new JPopupMenu();
		m_extractfit = new JMenuItem("Extract fitting traces");
		m_extractfit.setActionCommand("Extract fitting traces");
		m_extractfit.addActionListener(new ActionListener() {
	        @Override
	        public void actionPerformed(ActionEvent evt) {
	        	if(evt.getActionCommand().equals("Extract fitting traces")){
	        		ArrayList<ConstraintDefinition> cda = new ArrayList<ConstraintDefinition>();
	        		for(ConstraintDefinition cd : decModel.getModelCh().getConstraintDefinitions()){
	        			if(cd.getParameters().size() > 1 ){
		        			Rectangle2D bounds = decModel.getViewCh().getConnector(cd).getBounds();
		        			if(clickedBounds.contains(bounds)){
		        				cda.add(cd);
		        			}
	        			}
	        		}
	        		ArrayList<Integer> boundsXList = new ArrayList<Integer>();
	        		for(int as = 0 ; as < cda.size() ; as++){
	        			ConstraintDefinition cd = cda.get(as);
	        			Rectangle2D bounds = decModel.getViewCh().getConnector(cd).getBounds();
	        			boundsXList.add((int) bounds.getMinX());
	        		}
	        		
	        		int boundsX = (int) clickedBounds.getMinX();
	        		int closestX = closest(boundsX,boundsXList);
	        		int index = 0;
	        		for(int as = 0 ; as < boundsXList.size() ; as++){
	        			if(boundsXList.get(as) == closestX){
	        				index = as;
	        			}
	        		}
	        		ConstraintDefinition required = cda.get(index);
	        		org.processmining.plugins.declareminer.visualizing.ConstraintDefinition consDef = constraintMap.get(required);
	        		
	    			Parameter paramActivation = (Parameter) consDef.getParameters().toArray()[0];
	    			Parameter paramTarget = (Parameter) consDef.getParameters().toArray()[1];
	    			String activation = consDef.getBranches(paramActivation).iterator().next().getName(); // CDarray[i].getParameters().toArray()[0].toString();
	    			String constraint = consDef.getName();
	    			String target = consDef.getBranches(paramTarget).iterator().next().getName(); //CDarray[i].getParameters().toArray()[1].toString();
	    			String combined = activation + " " + constraint + " " + target;
	    			
	    			
	        		ArrayList<String> failedList = failedTracesMap.get(consDef);
	        		XLog fulfillingSet = getFulfillingSubLog(failedList);
        			publish(cont, "Cluster of fulfilled traces for " + combined, fulfillingSet, XLog.class, true);
        			System.out.println("Cluster of fulfilled traces for " + combined);
	        	}
	        }
	    });
		//m_extractfit.addMouseListener(this);
		m_extractnotfit = new JMenuItem("Extract violating traces");
		m_extractnotfit.setActionCommand("Extract violating traces");
		m_extractnotfit.addActionListener(new ActionListener() {
	        @Override
	        public void actionPerformed(ActionEvent evt) {
	        	if(evt.getActionCommand().equals("Extract violating traces")){
	        		ArrayList<ConstraintDefinition> cda = new ArrayList<ConstraintDefinition>();
	        		for(ConstraintDefinition cd : decModel.getModelCh().getConstraintDefinitions()){
	        			if(cd.getParameters().size() > 1 ){
		        			Rectangle2D bounds = decModel.getViewCh().getConnector(cd).getBounds();
		        			if(clickedBounds.contains(bounds)){
		        				cda.add(cd);
		        			}
	        			}
	        		}
	        		ArrayList<Integer> boundsXList = new ArrayList<Integer>();
	        		for(int as = 0 ; as < cda.size() ; as++){
	        			ConstraintDefinition cd = cda.get(as);
	        			Rectangle2D bounds = decModel.getViewCh().getConnector(cd).getBounds();
	        			boundsXList.add((int) bounds.getMinX());
	        		}
	        		
	        		int boundsX = (int) clickedBounds.getMinX();
	        		int closestX = closest(boundsX,boundsXList);
	        		int index = 0;
	        		for(int as = 0 ; as < boundsXList.size() ; as++){
	        			if(boundsXList.get(as) == closestX){
	        				index = as;
	        			}
	        		}
	        		ConstraintDefinition required = cda.get(index);
	        		

	        		org.processmining.plugins.declareminer.visualizing.ConstraintDefinition consDef = constraintMap.get(required);
	        		
	    			Parameter paramActivation = (Parameter) consDef.getParameters().toArray()[0];
	    			Parameter paramTarget = (Parameter) consDef.getParameters().toArray()[1];
	    			String activation = consDef.getBranches(paramActivation).iterator().next().getName(); // CDarray[i].getParameters().toArray()[0].toString();
	    			String constraint = consDef.getName();
	    			String target = consDef.getBranches(paramTarget).iterator().next().getName(); //CDarray[i].getParameters().toArray()[1].toString();
	    			String combined = activation + " " + constraint + " " + target;
	    			
	    			
	        		ArrayList<String> failedList = failedTracesMap.get(consDef);
	        		System.out.println(failedList.size());
	        		XLog violatingSet = getFailingSubLog(failedList);
        			publish(cont, "Cluster of violating traces for " + combined, violatingSet, XLog.class, true);
        			System.out.println("Cluster of violating traces for " + combined);
	        	}
	        }
	    });
	    m_nodeMenu.add(m_extractfit);
	    m_nodeMenu.addSeparator();
	    m_nodeMenu.add(m_extractnotfit);
	    MouseListener popupListener = new PopupListener(m_nodeMenu);
	    graph.addMouseListener(popupListener);
	}
	
	public int closest(int of, ArrayList<Integer> in) {
	    int min = Integer.MAX_VALUE;
	    int closest = of;

	    for (int v : in) {
	        final int diff = Math.abs(v - of);

	        if (diff < min) {
	            min = diff;
	            closest = v;
	        }
	    }

	    return closest;
	}
	
//	public static int closest2(int find, ArrayList<Integer> values) {
//	    int closest = values.get(0);
//	    int distance = Math.abs(closest - find);
//	    for(int i = 1 ; i < values.size() ; i++) {
//	       int distanceI = Math.abs(values.get(i) - find);
//	       if(distance > distanceI) {
//	           closest = values.get(i);
//	           distance = distanceI;
//	       }
//	    }
//	    return closest;
//	}
	
	private static Color getColor(Double value){
	    double green = 0;
	    double blue = 0;
	    double red = 0;
	    double intensity = value;
	    red = 3D * ((Math.min(intensity, 0.33333333)));
	    intensity = Math.max(intensity - 0.33333333, 0);
	    green = 3D * ((Math.min(intensity, 0.33333333)));
	    intensity = Math.max(intensity - 0.33333333, 0);
	    blue = 3D * ((Math.min(intensity, 0.33333333)));
	    return new Color((float) red, (float) green, (float) blue);
		
	}


  
    
	protected List<DefaultGraphCell> getCells(final Object object) {
		final List<DefaultGraphCell> list = new ArrayList<DefaultGraphCell>();
		DefaultGraphCell cell = null;
		for (int i = 0; i < graph.getModel().getRootCount(); i++) {
			final Object cellObject = graph.getModel().getRootAt(i);
			if (cellObject instanceof DefaultGraphCell) {
				cell = (DefaultGraphCell) cellObject;
				final Object userObject = cell.getUserObject();
				if (userObject.equals(object)) {
					list.add(cell);
				}
			}
		}
		return list;
	}
	
	private XLog getFailingSubLog(ArrayList<String> inputList){
		XFactory factory = new XFactoryNaiveImpl();
		XLog log = usedLog;
		XLog newLog = factory.createLog(log.getAttributes());
		
		for(XTrace oldTrace : log){
			if (inputList.contains(getName(oldTrace))){
				newLog.add(oldTrace);
			}		
		}
		return newLog;
	}
	
	private XLog getFulfillingSubLog(ArrayList<String> inputList){
		XFactory factory = new XFactoryNaiveImpl();
		XLog log = usedLog;
		XLog newLog = factory.createLog(log.getAttributes());
		
		for(XTrace oldTrace : log){
			if (!inputList.contains(getName(oldTrace))){
				newLog.add(oldTrace);
			}		
		}
		return newLog;
	}
	
	public static String getName(XAttributable element) {
		XAttributeLiteral name = (XAttributeLiteral) element.getAttributes().get("concept:name");
		return name.getValue();
	}
	
	private static <T> void publish(final UIPluginContext context, final String name, final T object, final Class<? super T> clazz, final boolean favorite) {
		final ProvidedObjectID id = context.getProvidedObjectManager().createProvidedObject(name, object, clazz, context);
			final GlobalContext gcontext = ((UIPluginContext) context).getGlobalContext();
			if (gcontext instanceof UIContext) {
				final UIContext uicontext = (UIContext) gcontext;
				final ResourceType resType = uicontext.getResourceManager().getResourceTypeFor(clazz);
				if (resType != null) {
					ProMTask task = null;
					try {
						final Field taskField = context.getClass().getDeclaredField("task");
						taskField.setAccessible(true);
						task = (ProMTask) taskField.get(context);
					} catch (final Exception _) {
						// Guess it wasn't meant to be, then...
					}
					final List<Collection<ProMPOResource>> lst = Collections.emptyList();
					ProMPOResource res = new ProMPOResource(uicontext, task == null ? null : task.getAction(), resType,
							id, lst);
					res = uicontext.getResourceManager().addResource(id, res);
				}
			}
		
		if (favorite) {
			ProvidedObjectHelper.setFavorite(context, object);
		}
	}
	
    private class PopupListener extends MouseAdapter {

        private JPopupMenu popup;

        PopupListener(JPopupMenu popupMenu) {
            popup = popupMenu;
        }

        @Override
        public void mouseClicked(MouseEvent e) {
    		if (SwingUtilities.isRightMouseButton(e) && e.getClickCount() == 1) {
    			if(graph.getModel().isEdge(graph.getSelectionCell())){
    				 Object v = graph.getSelectionCell();
    				 clickedBounds = (Rectangle) graph.getCellBounds(v);
    				 popup.show(e.getComponent(), e.getX(), e.getY());
    			}
    			
    		}
    	
        }
 
    }
}