package org.processmining.plugins.DeclareMapWithData.models;

import java.lang.ref.SoftReference;
import java.text.DateFormat;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class NumericValueConversion {
	public static double convertToFloat(String value)
	{
		double base=Math.pow(255, 2);
		double asNumber=0;
		for(int i=0;i<value.length();i++) {
			
			if (value.charAt(i)<=255)
				asNumber+=value.charAt(i)*(base);
			else
				asNumber+=base;
			base/=255; 
		}
		return asNumber;
	}
	
	private static final ThreadLocal<SoftReference<DateFormat>> THREAD_LOCAL_DF = new ThreadLocal<SoftReference<DateFormat>>();

	/**
	 * @return a DateFormat for each Thread 
	 */
	private static DateFormat getDateFormat() {
		SoftReference<DateFormat> softReference = THREAD_LOCAL_DF.get();
		if (softReference != null) {
			DateFormat dateFormat = softReference.get();
			if (dateFormat != null) {
				return dateFormat;
			}
		}
		DateFormat result = new SimpleDateFormat("EEE MMM dd kk:mm:ss zzz yyyy", Locale.US);
		softReference = new SoftReference<DateFormat>(result);
		THREAD_LOCAL_DF.set(softReference);
		return result;
	}
	
	public static double toNumericValue(Object value) {
		if (value instanceof Boolean) 
			return((Boolean) value ? 1 : 0);
		else if (value instanceof Number)
			return ((Number)value).doubleValue();
		else if (value instanceof String)
		{
            String target = (String) value;
            DateFormat df = getDateFormat();
         		
            // Avoid having to catch ParseException speeds up this conversion
            ParsePosition parsePosition = new ParsePosition(0);
            Date result =  df.parse(target, parsePosition);

			if (result != null) {
				return result.getTime();
			} else {
				return convertToFloat((String) value);
			}
            
		}
		else if (value instanceof Date)
			//TODO why does this divide by 1000 ??
			return ((Date)value).getTime()/1000D;
		else
			return 0;
	}

	public static Object fromNumericValue(double value,Class<?> type)
	{
		if (type == Boolean.class)
			return (value == 0 ? false : true);		
		if (type == Integer.class)
			return ((int)value);
		if (type == Long.class)
			return ((long)value);
		if (type == Double.class)
			return roundThreeDecimals(value);
		if (type == Float.class)
			return roundThreeDecimals((float)value);
		if (type == Date.class)
			//TODO why does this multiply by 1000 ??
			return new java.util.Date((long)(value*1000));
		if (type != String.class)
			return value;


		StringBuffer retValue=new StringBuffer();
		long intPart=(long)value;
		String valueAsString = String.valueOf(value);
		double decimalPart = Double.parseDouble("0." + valueAsString.split("\\.")[1]);

		while(intPart>0)
		{
			char module=(char) (intPart % 255);
			if (module>=32)
				retValue.insert(0,module);
			intPart/=255;
		}
		int iteration=1;
		while(iteration<15 && decimalPart>0)
		{
			decimalPart=decimalPart*255;
			if (decimalPart<32)
				break;
			Double rounded = (double)Math.round(decimalPart);
			retValue.append((char)rounded.intValue());
			decimalPart = decimalPart - rounded.intValue();
		}
		return retValue.toString();
	}

	
	private static double roundThreeDecimals(double d) {
		double value=(int) (d*1000);
		return value/1000D;
	}	

	private static float roundThreeDecimals(float d) {
		float value=(int) (d*1000);
		return value/1000F;
	}		
}
