package org.processmining.plugins.DeclareMapWithData.plugins;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;
import org.processmining.models.guards.Expression;
import org.processmining.plugins.DeclareMapWithData.dialogs.AddOperationsAndGuardsPanel;
import org.processmining.plugins.DeclareMapWithData.dialogs.AddVariablesPanel;
import org.processmining.plugins.DeclareMapWithData.models.DataDeclareMap;
import org.processmining.plugins.DeclareMapWithData.models.DataDeclareTemplate;
import org.processmining.plugins.declareminer.visualizing.ActivityDefinition;
import org.processmining.plugins.declareminer.visualizing.AssignmentModel;
import org.processmining.plugins.declareminer.visualizing.AssignmentModelView;
import org.processmining.plugins.declareminer.visualizing.ConstraintDefinition;
import org.processmining.plugins.declareminer.visualizing.ConstraintTemplate;
import org.processmining.plugins.declareminer.visualizing.DGraph;
import org.processmining.plugins.declareminer.visualizing.DeclareMap;
import org.processmining.plugins.declareminer.visualizing.IItem;
import org.processmining.plugins.declareminer.visualizing.Language;
import org.processmining.plugins.declareminer.visualizing.LanguageGroup;
import org.processmining.plugins.declareminer.visualizing.Parameter;
import org.processmining.plugins.declareminer.visualizing.TemplateBroker;
import org.processmining.plugins.declareminer.visualizing.XMLBrokerFactory;

import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.organic.JGraphOrganicLayout;

/*
 * Author: M.H.M. Schouten
 * E-mail: m.h.m.schouten0@gmail.com
 * 
 * This plugin allows for adding data to Declare models or editing existing Declare models made with this plugin. The user can first specify a 
 * list of data types. After this the user can specify guards  * for each constraint template in the Declare model. For each of these constraint 
 * templates with guards, one task is the activator which checks if the conditions holds.
 * 
 * Finally the Declare model is output with the guards shown on each constraint template enriched with guards.
 * 
 */
@Plugin(name = "Create/Edit DeclareMap With Data", parameterLabels = { "Input DeclareMap"  }, returnLabels = { "DataDeclareMap enriched with data"  }, returnTypes = { DataDeclareMap.class }, userAccessible = true)
public class EnrichDeclareModel  { //extends YourAlgorithm

	/**
	 * The plug-in variant that runs in any context and requires a parameters.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @return The output.
	 */
	
	// Each constraint template of the Declare model is stored in an array. 
	// It was chosen to use an array to ensure a fixed position is know for each constraint.
	// The name of the constraint and the guard is stored at the same position, where the name list the constraint and the connected tasks.
	ConstraintDefinition[] CDarray;
	String[] CDNamearray;
	String[] guards;
	Expression[] guardsExpression;

	@UITopiaVariant(affiliation = "TU/e", author = "M.H.M. Schouten", email = "m.h.m.schouten0@gmail.com")
	@PluginVariant(variantLabel = "Create DeclareMap with Data", requiredParameterLabels = { 0 })
	public DataDeclareMap run(UIPluginContext context, DeclareMap input) throws ParseException {

		int cdsize = input.getModel().getConstraintDefinitions().size();
		CDarray = new ConstraintDefinition[cdsize];
		CDNamearray = new String[cdsize];
		
		ConstraintDefinition[] CDarray2 = new ConstraintDefinition[cdsize];
		ConstraintDefinition[] additional2 = new ConstraintDefinition[cdsize];
		
		int i = 0;
		int k = 0;

		//The following mappings allow for linking DeclareTemplate names with the new DataDeclareTemplate names
		Map<String, DataDeclareTemplate> templateNameStringDataDeclareTemplateMap = new HashMap<String, DataDeclareTemplate>();
		Map<DataDeclareTemplate, ConstraintTemplate> DataDeclareTemplateConstraintTemplateMap = new HashMap<DataDeclareTemplate, ConstraintTemplate>();
		Map<DataDeclareTemplate, org.processmining.plugins.declare.visualizing.ConstraintTemplate> DataDeclareTemplateConstraintTemplateMap2 = new HashMap<DataDeclareTemplate, org.processmining.plugins.declare.visualizing.ConstraintTemplate>();
		
		//Since each activity should only be present ones, it is stored whether the activity is already present
		Vector<String> activityDefinitionsToShow = new Vector<String>();
		Vector<String> activityDefinitionsAlreadyAdded = new Vector<String>();
		
		//For each imported constraint template, the name and template is stored
		for(ConstraintDefinition constraintDefinition : input.getModel().getConstraintDefinitions()){
			if(constraintDefinition.getParameters().size() > 1 ){
				//CDarray[i] = constraintDefinition;
				CDarray2[i] = constraintDefinition;
				
				Parameter paramActivation = (Parameter) constraintDefinition.getParameters().toArray()[0];
				Parameter paramTarget = (Parameter) constraintDefinition.getParameters().toArray()[1];
				
				String activation = CDarray2[i].getBranches(paramActivation).iterator().next().getName(); // CDarray[i].getParameters().toArray()[0].toString();
				String constraint = CDarray2[i].getDisplay();
				String target = CDarray2[i].getBranches(paramTarget).iterator().next().getName(); //CDarray[i].getParameters().toArray()[1].toString();
				String combined = activation + " " + constraint + " " + target;
				
				CDNamearray[i] = combined;
				i++;
			//mapping.put(combined , constraintDefinition);
			}
			else{
				additional2[k] = constraintDefinition;
				k++;
			}
			
			
		}
		CDarray = new ConstraintDefinition[i];
		for(int l = 0; l < i;l++){
			CDarray[l] = CDarray2[l];
		}
		ConstraintDefinition[] additional = new ConstraintDefinition[k];
		for(int l = 0; l < k;l++){
			additional[l] = additional2[l];
		}
		
		//Necessary for gathering the correct DataDeclareTemplates
		DataDeclareTemplate declareTemplate = DataDeclareTemplate.Alternate_Precedence;
		DataDeclareTemplate[] declareTemplateNames = declareTemplate.getDeclaringClass().getEnumConstants();
		for(DataDeclareTemplate d : declareTemplateNames){
			String templateNameString = d.toString().replaceAll("_", " ").toLowerCase();
			if(templateNameString.equals("not ch precedence")){
				templateNameString = "not chain precedence";
			}
			if(templateNameString.equals("not ch response")){
				templateNameString = "not chain response";
			}
			if(templateNameString.equals("not ch succession")){
				templateNameString = "not chain succession";
			}
			templateNameStringDataDeclareTemplateMap.put(templateNameString, d);
		}
		
		//Each constraint and activity added to the output DataDeclareMap gets a unique id
		int activityID = 0;
		int constraintID = 0;
		
		String arg1 = "";
		String arg2 = "";
		
		//Every constraint has an activiation and a target
		ActivityDefinition activitydefinitionActivation = null;
		ActivityDefinition activitydefinitionTarget = null;
		
		// At the moment two packages are present "declare" and "declareminer", the latter is most recent
		// Unfortunately it is not possible to move the constraints in the output model when only using the latter.
		// As a result the storing of activities and constraints is done twice, once for "declare" and once for "declareminer"
		// In the end only the graphical results of "declare" are used
		org.processmining.plugins.declare.visualizing.ActivityDefinition activitydefinitionActivation2 = null;
		org.processmining.plugins.declare.visualizing.ActivityDefinition activitydefinitionTarget2 = null;
		
		//The following code is required for getting the condec language 
		InputStream templateInputStream = ClassLoader.getSystemClassLoader().getResourceAsStream("resources/template.xml");
		File languageFile = null;
		try {
			languageFile = File.createTempFile("template", ".xml");
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(templateInputStream));
			String line = bufferedReader.readLine();
			PrintStream out = new PrintStream(languageFile);
			while (line != null) {
				out.println(line);
				line = bufferedReader.readLine();
			}
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		TemplateBroker template = XMLBrokerFactory.newTemplateBroker(languageFile.getAbsolutePath());
		List<Language> languages = template.readLanguages();
		Language lang = languages.get(0);
		//Duplicate for "declare"
		org.processmining.plugins.declare.visualizing.TemplateBroker template2 = org.processmining.plugins.declare.visualizing.XMLBrokerFactory.newTemplateBroker(languageFile.getAbsolutePath());
		List<org.processmining.plugins.declare.visualizing.Language> languages2 = template2.readLanguages();
		org.processmining.plugins.declare.visualizing.Language lang2 = languages2.get(0);
		
		//Create the new DataDeclareMap which will be used as output
		AssignmentModel model = new AssignmentModel(lang);
		org.processmining.plugins.declare.visualizing.AssignmentModel modelCh = new org.processmining.plugins.declare.visualizing.AssignmentModel(lang2); 
		org.processmining.plugins.declare.visualizing.AssignmentModelView viewCh = new org.processmining.plugins.declare.visualizing.AssignmentModelView(modelCh);
		model.setName("new model");
		modelCh.setName("new model");
		AssignmentModelView view = new AssignmentModelView(model);
		DataDeclareMap map = new DataDeclareMap(model,view,null, modelCh, viewCh, null); 
		
		// This creates the dialog screen used to specify data types
		AddVariablesPanel varPanel = new AddVariablesPanel();
		InteractionResult result = context.showConfiguration("Configuration", varPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		// Each specified data type is added to the DataDeclareMap
		for(String elem : varPanel.getDataElements2()){
			map.addVariable(elem);
		}
		
		// This creates the dialog screen used to specify guards
		AddOperationsAndGuardsPanel guardPanel = new AddOperationsAndGuardsPanel(CDNamearray, map.getVariables()); //, map.getVariables()
		result = context.showConfiguration("Configuration", guardPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		// Guards are stored as String and as Expression
		// The Expression format allows for checking the guards for correctness, necessary for conformance checking
		guards = guardPanel.getGuardsStrings();
		guardsExpression = guardPanel.getGuards();
		
		//the first language in the list is the condec language, which is what we need
		List<Language> languagesList = template.readLanguages();
		Language condecLanguage = languagesList.get(0);
		List<IItem> templateList = new ArrayList<IItem>();
		
		//Fill the DataDeclareTemplate map, both for "declare"and "declareminer" objects
		List<IItem> condecLanguageChildrenList = condecLanguage.getChildren();
		for (IItem condecLanguageChild : condecLanguageChildrenList) {
			if (condecLanguageChild instanceof LanguageGroup) {
				templateList.addAll(visit(condecLanguageChild));
			} else {
				templateList.add(condecLanguageChild);
			}
		}
		
		List<org.processmining.plugins.declare.visualizing.Language> languagesList2 = template2.readLanguages();
		org.processmining.plugins.declare.visualizing.Language condecLanguage2 = languagesList2.get(0);
		List<org.processmining.plugins.declare.visualizing.IItem> templateList2 = new ArrayList<org.processmining.plugins.declare.visualizing.IItem>();
		List<org.processmining.plugins.declare.visualizing.IItem> condecLanguageChildrenList2 = condecLanguage2.getChildren();
		for (org.processmining.plugins.declare.visualizing.IItem condecLanguageChild : condecLanguageChildrenList2) {
			if (condecLanguageChild instanceof org.processmining.plugins.declare.visualizing.LanguageGroup) {
				templateList2.addAll(visit2(condecLanguageChild));
			} else {
				templateList2.add(condecLanguageChild);
			}
		}
		
		for(IItem item : templateList){
			if(item instanceof ConstraintTemplate){
				ConstraintTemplate constraintTemplate = (ConstraintTemplate)item;

				if(templateNameStringDataDeclareTemplateMap.containsKey(constraintTemplate.getName().replaceAll("-", " ").toLowerCase())){//.replaceAll("-", " ")
					if(!constraintTemplate.getName().contains("co-existence")){
						DataDeclareTemplateConstraintTemplateMap.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
						
					}
					if(constraintTemplate.getName().contains("co-existence")){
						DataDeclareTemplateConstraintTemplateMap.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
					}
				}
			}
		}
		
		for(org.processmining.plugins.declare.visualizing.IItem item : templateList2){
			if(item instanceof org.processmining.plugins.declare.visualizing.ConstraintTemplate){
				org.processmining.plugins.declare.visualizing.ConstraintTemplate constraintTemplate = (org.processmining.plugins.declare.visualizing.ConstraintTemplate)item;
				if(templateNameStringDataDeclareTemplateMap.containsKey(constraintTemplate.getName().replaceAll("-", " ").toLowerCase())){ //.replaceAll("-", "") na name
					if(!constraintTemplate.getName().contains("co-existence")){	
						DataDeclareTemplateConstraintTemplateMap2.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
					}
					if(constraintTemplate.getName().contains("co-existence")){	
						DataDeclareTemplateConstraintTemplateMap2.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
					}	
				}
			}
		}

		int p = 0;
		List<String> parameters2 = new ArrayList<String>();
		
		// For each constraint template in the imported DeclareMap it is checked whether a guard is specified
		// If so the template is enriched with data
		int length = i;
		for(i = 0; i < length; i++){
			// Precedence related templates (Alternate, Chain, Not) the activation is the second activity, i.e. B is the activation in Precedence(A,B)
			// As a result a distinction is made in these templates
			DataDeclareTemplate switchVariable = getTemplate(CDarray[i]);
			if(switchVariable.toString().contains("Precedence")){
				switchVariable = DataDeclareTemplate.Precedence;
			}
			else {
				switchVariable = DataDeclareTemplate.Response;
			}
			
			// In this case the first activity is the activation, i.e. A is the activation in Response(A,B)
			switch(switchVariable){	
				case Response: {
					// Get the activities linked to the constraint
					for(Parameter parameter : CDarray[i].getParameters()){
						if(CDarray[i].getBranches(parameter).iterator().hasNext()){
							String activityName = CDarray[i].getBranches(parameter).iterator().next().getName();
							if((activityName.contains("-assign")||activityName.contains("-ate_abort")||activityName.contains("-suspend")||activityName.contains("-complete")||activityName.contains("-autoskip")||activityName.contains("-manualskip")||activityName.contains("pi_abort")||activityName.contains("-reassign")||activityName.contains("-resume")||activityName.contains("-schedule")||activityName.contains("-start")||activityName.contains("-unknown")||activityName.contains("-withdraw"))){
								String[] splittedName = CDarray[i].getBranches(parameter).iterator().next().getName().split("-");
								activityName = splittedName[0];
								for(int j = 1; i<splittedName.length-1; i++){
									activityName = activityName + "-" + splittedName[j];
								}	
							}
							parameters2.add(activityName);
						}else{
							parameters2.add("EMPTY_PARAM");
						}
						p++;
						
					}
					
					//The names of the activities are set correctly
					arg1 = parameters2.get(p-2);
					arg2 = parameters2.get(p-1);
					
					//Determine whether the activity is already present in the output model
					if(!activityDefinitionsAlreadyAdded.contains(arg1)){
						activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionActivation.setName(arg1);
						activitydefinitionActivation2.setName(arg1);
						activityDefinitionsAlreadyAdded.add(arg1);
						activityID ++;
					}else{
						activitydefinitionActivation = model.activityDefinitionWithName(arg1);
						activitydefinitionActivation2 = modelCh.activityDefinitionWithName(arg1);
					}
					
					if(!activityDefinitionsAlreadyAdded.contains(arg2)){
						activityDefinitionsAlreadyAdded.add(arg2);
						activitydefinitionTarget = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionTarget2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionTarget.setName(arg2);
						activitydefinitionTarget2.setName(arg2);
						activityID ++;
					}else{
						activitydefinitionTarget = model.activityDefinitionWithName(arg2);
						activitydefinitionTarget2 = modelCh.activityDefinitionWithName(arg2);
					}
					
					//Get the corresponding DataDeclreTemplate of the current DeclareTemplate
					DataDeclareTemplate temp = getTemplate(CDarray[i]);					
					ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, DataDeclareTemplateConstraintTemplateMap.get(temp));
					org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID, modelCh, DataDeclareTemplateConstraintTemplateMap2.get(temp));
					
					Collection<Parameter> parameters = (DataDeclareTemplateConstraintTemplateMap.get(temp)).getParameters();
					Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = (DataDeclareTemplateConstraintTemplateMap2.get(temp)).getParameters();
					
					//Correctly set which activity is the activation
					constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionActivation);
					constraintdefinition.addBranch((Parameter) parameters.toArray()[1], activitydefinitionTarget);
					
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionActivation2);
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[1], activitydefinitionTarget2);

					//Update the visualization label in case the constraint has a guard
					if(guards[i] != "") {
						String displaylabel = constraintdefinition.getDisplay();
						constraintdefinition.setDisplay(displaylabel + " " + guards[i]);
						constraintdefinition2.setDisplay(displaylabel + " " + guards[i]);
					}
					
					//Add the new data constraint
					model.addConstraintDefiniton(constraintdefinition);
					modelCh.addConstraintDefiniton(constraintdefinition2);
					
					//Determine if the activity is already present in the model
					if(!activityDefinitionsToShow.contains(arg1)){
						activityDefinitionsToShow.add(arg1);
					}
					if(!activityDefinitionsToShow.contains(arg2)){
						activityDefinitionsToShow.add(arg2);
					}
					
					CDarray[i] = constraintdefinition;
					
					//Update the id's for the next iteration
					activityID++;
					constraintID++;
					break; }
				case Precedence: {	
					//This case works in a similar fashion
					for(Parameter parameter : CDarray[i].getParameters()){
						if(CDarray[i].getBranches(parameter).iterator().hasNext()){
							String activityName = CDarray[i].getBranches(parameter).iterator().next().getName();
							if((activityName.contains("-assign")||activityName.contains("-ate_abort")||activityName.contains("-suspend")||activityName.contains("-complete")||activityName.contains("-autoskip")||activityName.contains("-manualskip")||activityName.contains("pi_abort")||activityName.contains("-reassign")||activityName.contains("-resume")||activityName.contains("-schedule")||activityName.contains("-start")||activityName.contains("-unknown")||activityName.contains("-withdraw"))){
								String[] splittedName = CDarray[i].getBranches(parameter).iterator().next().getName().split("-");
								activityName = splittedName[0];
								for(int j = 1; i<splittedName.length-1; i++){
									activityName = activityName + "-" + splittedName[j];
								}	
							}
							parameters2.add(activityName);
						}else{
							parameters2.add("EMPTY_PARAM");
						}
						p++;
						
					}
					
					arg1 = parameters2.get(p-1);
					arg2 = parameters2.get(p-2);
					
					if(!activityDefinitionsAlreadyAdded.contains(arg1)){
						activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionActivation.setName(arg1);
						activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionActivation2.setName(arg1);
						activityDefinitionsAlreadyAdded.add(arg1);
						activityID ++;
					}else{
						activitydefinitionActivation = model.activityDefinitionWithName(arg1);
						activitydefinitionActivation2 = modelCh.activityDefinitionWithName(arg1);
					}
					
					if(!activityDefinitionsAlreadyAdded.contains(arg2)){
						activityDefinitionsAlreadyAdded.add(arg2);
						activitydefinitionTarget = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionTarget.setName(arg2);
						activitydefinitionTarget2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionTarget2.setName(arg2);
						activityID ++;
					}else{
						activitydefinitionTarget = model.activityDefinitionWithName(arg2);
						activitydefinitionTarget2 = modelCh.activityDefinitionWithName(arg2);
					}
					
					DataDeclareTemplate temp = getTemplate(CDarray[i]);
					ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, DataDeclareTemplateConstraintTemplateMap.get(temp));
					org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID, modelCh, DataDeclareTemplateConstraintTemplateMap2.get(temp));
					
					Collection<Parameter> parameters = (DataDeclareTemplateConstraintTemplateMap.get(temp)).getParameters();
					Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = (DataDeclareTemplateConstraintTemplateMap2.get(temp)).getParameters();
					
					constraintdefinition.addBranch((Parameter) parameters.toArray()[1], activitydefinitionActivation);
					constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionTarget);
		
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[1], activitydefinitionActivation2);
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionTarget2);
		
					if(guards[i] != ""|| !guards[i].trim().isEmpty()) {
						String displaylabel = constraintdefinition.getDisplay();
						constraintdefinition.setDisplay(displaylabel + " " + guards[i]);
						constraintdefinition2.setDisplay(displaylabel + " " + guards[i]);
					}
					
		
					model.addConstraintDefiniton(constraintdefinition);
					modelCh.addConstraintDefiniton(constraintdefinition2);
					
					if(!activityDefinitionsToShow.contains(arg1)){
						activityDefinitionsToShow.add(arg1);
					}
					if(!activityDefinitionsToShow.contains(arg2)){
						activityDefinitionsToShow.add(arg2);
					}
					
					CDarray[i] = constraintdefinition;
					
					activityID++;
					constraintID++;
					break; }

			}	
			
		}	
		
		//Add the constraints which cannot be enriched with guards
		if(additional.length > 0){
			for(i = 0; i < additional.length; i++){
				if(additional[i].getParameters().size() < 2){
					for(Parameter parameter : additional[i].getParameters()){
						String activityName = additional[i].getBranches(parameter).iterator().next().getName();
						if(!activityDefinitionsAlreadyAdded.contains(activityName)){
							activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
							activitydefinitionActivation.setName(activityName);
							activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
							activitydefinitionActivation2.setName(activityName);
							activityDefinitionsAlreadyAdded.add(activityName);
							activityID ++;
						}else{
							activitydefinitionActivation = model.activityDefinitionWithName(activityName);
							activitydefinitionActivation2 = modelCh.activityDefinitionWithName(activityName);
						}
						String constraintName = additional[i].getName();
						String constraintDisplay = additional[i].getDisplay();
						String constraintDescription = additional[i].getDescription();
						String constraintText = additional[i].getText();
						org.processmining.plugins.declare.visualizing.Parameter param1 = new org.processmining.plugins.declare.visualizing.Parameter(0, activityName);
						Parameter param2 = new Parameter(0, activityName);
						param1.setBranchable(true);
						param2.setBranchable(true);
						org.processmining.plugins.declare.visualizing.ConstraintTemplate dct = new org.processmining.plugins.declare.visualizing.ConstraintTemplate(i,lang2);
						ConstraintTemplate dct2 = new ConstraintTemplate(i,lang);
						dct.addParameter(param1);
						dct.setDescription(constraintDescription);
						dct.setDisplay(constraintDisplay);
						dct.setName(constraintName);
						dct.setText(constraintText);
						
						dct2.addParameter(param2);
						dct2.setDescription(constraintDescription);
						dct2.setDisplay(constraintDisplay);
						dct2.setName(constraintName);
						dct2.setText(constraintText);
						
						org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID,modelCh,dct);
						ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, dct2);
						Collection<Parameter> parameters = dct2.getParameters();
						Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = dct.getParameters();
						
						constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionActivation);
						constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionActivation2);
						model.addConstraintDefiniton(constraintdefinition);
						modelCh.addConstraintDefiniton(constraintdefinition2);
						//System.out.println(constraintdefinition.getName() + " " + constraintdefinition2.getName());
						//System.out.println(((Parameter) parameters.toArray()[0]).getName() + " " + ((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0]).getName());
						if(!activityDefinitionsToShow.contains(activityName)){
							activityDefinitionsToShow.add(activityName);
						}
						activityID++;
						constraintID++;
					}
				}/*
				else {
					// Get the activities linked to the constraint
					for(Parameter parameter : additional[i].getParameters()){
						if(additional[i].getBranches(parameter).iterator().hasNext()){
							String activityName = additional[i].getBranches(parameter).iterator().next().getName();
							if((activityName.contains("-assign")||activityName.contains("-ate_abort")||activityName.contains("-suspend")||activityName.contains("-complete")||activityName.contains("-autoskip")||activityName.contains("-manualskip")||activityName.contains("pi_abort")||activityName.contains("-reassign")||activityName.contains("-resume")||activityName.contains("-schedule")||activityName.contains("-start")||activityName.contains("-unknown")||activityName.contains("-withdraw"))){
								String[] splittedName = additional[i].getBranches(parameter).iterator().next().getName().split("-");
								activityName = splittedName[0];
								for(int j = 1; i<splittedName.length-1; i++){
									activityName = activityName + "-" + splittedName[j];
								}	
							}
							parameters2.add(activityName);
						}else{
							parameters2.add("EMPTY_PARAM");
						}
						p++;
						
					}
					
					//The names of the activities are set correctly
					arg1 = parameters2.get(p-2);
					arg2 = parameters2.get(p-1);
					
					//Determine whether the activity is already present in the output model
					if(!activityDefinitionsAlreadyAdded.contains(arg1)){
						activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionActivation.setName(arg1);
						activitydefinitionActivation2.setName(arg1);
						activityDefinitionsAlreadyAdded.add(arg1);
						activityID ++;
					}else{
						activitydefinitionActivation = model.activityDefinitionWithName(arg1);
						activitydefinitionActivation2 = modelCh.activityDefinitionWithName(arg1);
					}
					
					if(!activityDefinitionsAlreadyAdded.contains(arg2)){
						activityDefinitionsAlreadyAdded.add(arg2);
						activitydefinitionTarget = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionTarget2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionTarget.setName(arg2);
						activitydefinitionTarget2.setName(arg2);
						activityID ++;
					}else{
						activitydefinitionTarget = model.activityDefinitionWithName(arg2);
						activitydefinitionTarget2 = modelCh.activityDefinitionWithName(arg2);
					}
					String constraintName = additional[i].getName();
					String constraintDisplay = additional[i].getDisplay();
					String constraintDescription = additional[i].getDescription();
					String constraintText = additional[i].getText();
					
					org.processmining.plugins.declare.visualizing.Parameter param11 = new org.processmining.plugins.declare.visualizing.Parameter(0, arg1);
					org.processmining.plugins.declare.visualizing.Parameter param12 = new org.processmining.plugins.declare.visualizing.Parameter(1, arg2);
					Parameter param21 = new Parameter(0, arg1);
					Parameter param22 = new Parameter(1, arg2);
					param11.setBranchable(true);
					param21.setBranchable(true);
					param12.setBranchable(true);
					param22.setBranchable(true);
					
					org.processmining.plugins.declare.visualizing.ConstraintTemplate dct = new org.processmining.plugins.declare.visualizing.ConstraintTemplate(i,lang2);
					//org.processmining.plugins.declare.visualizing.ConstraintTemplate dct12 = new org.processmining.plugins.declare.visualizing.ConstraintTemplate(i+100,lang2);
					ConstraintTemplate dct2 = new ConstraintTemplate(i,lang);
					//ConstraintTemplate dct22 = new ConstraintTemplate(i+100,lang);
					
					dct.addParameter(param11);
					dct.addParameter(param12);
					dct.setDescription(constraintDescription);
					dct.setDisplay(constraintDisplay);
					dct.setName(constraintName);
					dct.setText(constraintText);
					
					dct2.addParameter(param21);
					dct2.addParameter(param22);
					dct2.setDescription(constraintDescription);
					dct2.setDisplay(constraintDisplay);
					dct2.setName(constraintName);
					dct2.setText(constraintText);
										
					ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, dct2);
					org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID, modelCh, dct);
					
					Collection<Parameter> parameters = dct2.getParameters();
					Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = dct.getParameters();
					
					//Correctly set which activity is the activation
					constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionActivation);
					constraintdefinition.addBranch((Parameter) parameters.toArray()[1], activitydefinitionTarget);
					
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionActivation2);
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[1], activitydefinitionTarget2);
					
					//Add the new data constraint
					model.addConstraintDefiniton(constraintdefinition);
					modelCh.addConstraintDefiniton(constraintdefinition2);
					
					//Determine if the activity is already present in the model
					if(!activityDefinitionsToShow.contains(arg1)){
						activityDefinitionsToShow.add(arg1);
					}
					if(!activityDefinitionsToShow.contains(arg2)){
						activityDefinitionsToShow.add(arg2);
					}
					
					//Update the id's for the next iteration
					activityID++;
					constraintID++;
				}*/

			}
		}
		
		//Remove duplicate activities
		for(String ad : activityDefinitionsAlreadyAdded){
			if(!activityDefinitionsToShow.contains(ad)){
				model.deleteActivityDefinition(model.activityDefinitionWithName(ad));
			}
		}
		
		view = new AssignmentModelView(model);
		viewCh = new org.processmining.plugins.declare.visualizing.AssignmentModelView(modelCh);
		
		// Improve visualization by giving for example by giving penalties to crossing edges
		DGraph graph = view.getGraph();
		org.processmining.plugins.declare.visualizing.DGraph graph2 = viewCh.getGraph();
		
		final JGraphOrganicLayout oc = new JGraphOrganicLayout();
		
		oc.setDeterministic(true);
		oc.setOptimizeBorderLine(true);
		oc.setOptimizeEdgeCrossing(true);
		oc.setOptimizeEdgeDistance(true);
		oc.setOptimizeEdgeLength(true);
		oc.setOptimizeNodeDistribution(true);
		oc.setEdgeCrossingCostFactor(999999999);
		oc.setEdgeDistanceCostFactor(999999999);
		oc.setFineTuning(true);
		
		oc.setEdgeLengthCostFactor(9999);
		oc.setEdgeLengthCostFactor(99);
		oc.setNodeDistributionCostFactor(999999999);
		oc.setBorderLineCostFactor(999);
		oc.setRadiusScaleFactor(0.9);
		final JGraphFacade jgf = new JGraphFacade(graph);
		oc.run(jgf);
		final Map nestedMap = jgf.createNestedMap(true, true); 
		graph.getGraphLayoutCache().edit(nestedMap); 
	
		final JGraphFacade jgf2 = new JGraphFacade(graph2);
		oc.run(jgf2);
		final Map nestedMap2 = jgf2.createNestedMap(true, true); 
		graph2.getGraphLayoutCache().edit(nestedMap2); 
			
		// Set all necessary output
		DataDeclareMap output = new DataDeclareMap(model,view,null, modelCh, viewCh, null);
		output.setCDarray(CDarray);
		output.setCDNamearray(CDNamearray);
		output.setGuards(guards);
		output.setVariables(map.getVariables());
	    return output;
	}
	
	// The edit plugin works in a similar fashion
	@UITopiaVariant(affiliation = "TU/e", author = "M.H.M. Schouten", email = "m.h.m.schouten0@gmail.com")
	@PluginVariant(variantLabel = "Edit DeclareMap with Data", requiredParameterLabels = { 0 })
	public DataDeclareMap run(UIPluginContext context, DataDeclareMap input) throws ParseException {
		// Read the information of old DataDeclareMap
		CDNamearray = input.getCDNamearray();
		guards = input.getGuards();
		
		ConstraintDefinition[] CDarray2 = new ConstraintDefinition[input.getModel().getConstraintDefinitions().size()];
		ConstraintDefinition[] additional2 = new ConstraintDefinition[input.getModel().getConstraintDefinitions().size()];
		
		int i = 0;
		int k = 0;
		
		//For each imported constraint template, the name and template is stored
		for(ConstraintDefinition constraintDefinition : input.getModel().getConstraintDefinitions()){
			if(constraintDefinition.getParameters().size() > 1 ){
				//CDarray[i] = constraintDefinition;
				CDarray2[i] = constraintDefinition;
				i++;
			//mapping.put(combined , constraintDefinition);
			}
			else{
				additional2[k] = constraintDefinition;
				k++;
			}
			
			
		}
		CDarray = new ConstraintDefinition[i];
		for(int l = 0; l < i;l++){
			CDarray[l] = CDarray2[l];
		}
		ConstraintDefinition[] additional = new ConstraintDefinition[k];
		for(int l = 0; l < k;l++){
			additional[l] = additional2[l];
		}
		
		
		Map<String, DataDeclareTemplate> templateNameStringDataDeclareTemplateMap = new HashMap<String, DataDeclareTemplate>();
		Map<DataDeclareTemplate, ConstraintTemplate> DataDeclareTemplateConstraintTemplateMap = new HashMap<DataDeclareTemplate, ConstraintTemplate>();
		Map<DataDeclareTemplate, org.processmining.plugins.declare.visualizing.ConstraintTemplate> DataDeclareTemplateConstraintTemplateMap2 = new HashMap<DataDeclareTemplate, org.processmining.plugins.declare.visualizing.ConstraintTemplate>();
		
		Vector<String> activityDefinitionsToShow = new Vector<String>();
		Vector<String> activityDefinitionsAlreadyAdded = new Vector<String>();

		DataDeclareTemplate declareTemplate = DataDeclareTemplate.Alternate_Precedence;
		DataDeclareTemplate[] declareTemplateNames = declareTemplate.getDeclaringClass().getEnumConstants();
		
		for(DataDeclareTemplate d : declareTemplateNames){
			String templateNameString = d.toString().replaceAll("_", " ").toLowerCase();
			if(templateNameString.equals("not ch precedence")){
				templateNameString = "not chain precedence";
			}
			if(templateNameString.equals("not ch response")){
				templateNameString = "not chain response";
			}
			if(templateNameString.equals("not ch succession")){
				templateNameString = "not chain succession";
			}
			templateNameStringDataDeclareTemplateMap.put(templateNameString, d);
		}
		
		int activityID = 0;
		int constraintID = 0;
		
		String arg1 = "";
		String arg2 = "";
		
		
		ActivityDefinition activitydefinitionActivation = null;
		ActivityDefinition activitydefinitionTarget = null;
		
		org.processmining.plugins.declare.visualizing.ActivityDefinition activitydefinitionActivation2 = null;
		org.processmining.plugins.declare.visualizing.ActivityDefinition activitydefinitionTarget2 = null;
		
		InputStream templateInputStream = ClassLoader.getSystemClassLoader().getResourceAsStream("resources/template.xml");
		File languageFile = null;
		try {
			languageFile = File.createTempFile("template", ".xml");
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(templateInputStream));
			String line = bufferedReader.readLine();
			PrintStream out = new PrintStream(languageFile);
			while (line != null) {
				out.println(line);
				line = bufferedReader.readLine();
			}
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		TemplateBroker template = XMLBrokerFactory.newTemplateBroker(languageFile.getAbsolutePath());
		List<Language> languages = template.readLanguages();
		Language lang = languages.get(0);
		
		org.processmining.plugins.declare.visualizing.TemplateBroker template2 = org.processmining.plugins.declare.visualizing.XMLBrokerFactory.newTemplateBroker(languageFile.getAbsolutePath());
		List<org.processmining.plugins.declare.visualizing.Language> languages2 = template2.readLanguages();
		org.processmining.plugins.declare.visualizing.Language lang2 = languages2.get(0);
		
		
		AssignmentModel model = new AssignmentModel(lang);
		org.processmining.plugins.declare.visualizing.AssignmentModel modelCh = new org.processmining.plugins.declare.visualizing.AssignmentModel(lang2); 
		org.processmining.plugins.declare.visualizing.AssignmentModelView viewCh = new org.processmining.plugins.declare.visualizing.AssignmentModelView(modelCh);
		model.setName("new model");
		modelCh.setName("new model");
		AssignmentModelView view = new AssignmentModelView(model);
		DataDeclareMap map = new DataDeclareMap(model,view,null, modelCh, viewCh, null); 
		map.setVariables(input.getVariables());
		map.setGuards(input.getGuards());
		
		AddVariablesPanel varPanel = new AddVariablesPanel(map);
		InteractionResult result = context.showConfiguration("Configuration", varPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		for(String elem : varPanel.getDataElements2()){
			map.addVariable(elem);
		}
		
		AddOperationsAndGuardsPanel guardPanel = new AddOperationsAndGuardsPanel(CDNamearray, map.getVariables(), map); //, map.getVariables()
		result = context.showConfiguration("Configuration", guardPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		guards = guardPanel.getGuardsStrings();
		guardsExpression = guardPanel.getGuards();

		List<Language> languagesList = template.readLanguages();
		//the first language in the list is the condec language, which is what we need
		Language condecLanguage = languagesList.get(0);
		List<IItem> templateList = new ArrayList<IItem>();
		List<IItem> condecLanguageChildrenList = condecLanguage.getChildren();
		for (IItem condecLanguageChild : condecLanguageChildrenList) {
			if (condecLanguageChild instanceof LanguageGroup) {
				templateList.addAll(visit(condecLanguageChild));
			} else {
				templateList.add(condecLanguageChild);
			}
		}
		
		List<org.processmining.plugins.declare.visualizing.Language> languagesList2 = template2.readLanguages();
		org.processmining.plugins.declare.visualizing.Language condecLanguage2 = languagesList2.get(0);
		List<org.processmining.plugins.declare.visualizing.IItem> templateList2 = new ArrayList<org.processmining.plugins.declare.visualizing.IItem>();
		List<org.processmining.plugins.declare.visualizing.IItem> condecLanguageChildrenList2 = condecLanguage2.getChildren();
		for (org.processmining.plugins.declare.visualizing.IItem condecLanguageChild : condecLanguageChildrenList2) {
			if (condecLanguageChild instanceof org.processmining.plugins.declare.visualizing.LanguageGroup) {
				templateList2.addAll(visit2(condecLanguageChild));
			} else {
				templateList2.add(condecLanguageChild);
			}
		}
		
		for(IItem item : templateList){
			if(item instanceof ConstraintTemplate){
				ConstraintTemplate constraintTemplate = (ConstraintTemplate)item;

				if(templateNameStringDataDeclareTemplateMap.containsKey(constraintTemplate.getName().replaceAll("-", " ").toLowerCase())){//.replaceAll("-", " ")
					if(!constraintTemplate.getName().contains("co-existence")){
						DataDeclareTemplateConstraintTemplateMap.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
						
					}
					if(constraintTemplate.getName().contains("co-existence")){
						DataDeclareTemplateConstraintTemplateMap.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
					}
				}
			}
		}
		
		for(org.processmining.plugins.declare.visualizing.IItem item : templateList2){
			if(item instanceof org.processmining.plugins.declare.visualizing.ConstraintTemplate){
				org.processmining.plugins.declare.visualizing.ConstraintTemplate constraintTemplate = (org.processmining.plugins.declare.visualizing.ConstraintTemplate)item;
				if(templateNameStringDataDeclareTemplateMap.containsKey(constraintTemplate.getName().replaceAll("-", " ").toLowerCase())){ //.replaceAll("-", "") na name
					if(!constraintTemplate.getName().contains("co-existence")){	
						DataDeclareTemplateConstraintTemplateMap2.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
					}
					if(constraintTemplate.getName().contains("co-existence")){	
						DataDeclareTemplateConstraintTemplateMap2.put(templateNameStringDataDeclareTemplateMap.get(constraintTemplate.getName().replaceAll("-", " ").toLowerCase()), constraintTemplate);
					}	
				}
			}
		}

		int p = 0;
		List<String> parameters2 = new ArrayList<String>();
		
		int length = i;
		for(i = 0; i < length; i++){
			
			DataDeclareTemplate switchVariable = getTemplate(CDarray[i]);
			
			if(switchVariable.toString().contains("Precedence")){
				switchVariable = DataDeclareTemplate.Precedence;
			}
			else {
				switchVariable = DataDeclareTemplate.Response;
			}
			
			
			switch(switchVariable){	
				case Response: {	
					for(Parameter parameter : CDarray[i].getParameters()){
						if(CDarray[i].getBranches(parameter).iterator().hasNext()){
							String activityName = CDarray[i].getBranches(parameter).iterator().next().getName();
							if((activityName.contains("-assign")||activityName.contains("-ate_abort")||activityName.contains("-suspend")||activityName.contains("-complete")||activityName.contains("-autoskip")||activityName.contains("-manualskip")||activityName.contains("pi_abort")||activityName.contains("-reassign")||activityName.contains("-resume")||activityName.contains("-schedule")||activityName.contains("-start")||activityName.contains("-unknown")||activityName.contains("-withdraw"))){
								String[] splittedName = CDarray[i].getBranches(parameter).iterator().next().getName().split("-");
								activityName = splittedName[0];
								for(int j = 1; i<splittedName.length-1; i++){
									activityName = activityName + "-" + splittedName[j];
								}	
							}
							parameters2.add(activityName);
						}else{
							parameters2.add("EMPTY_PARAM");
						}
						p++;
						
					}
					
					arg1 = parameters2.get(p-2);
					arg2 = parameters2.get(p-1);
					
					if(!activityDefinitionsAlreadyAdded.contains(arg1)){
						activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionActivation.setName(arg1);
						activitydefinitionActivation2.setName(arg1);
						activityDefinitionsAlreadyAdded.add(arg1);
						activityID ++;
					}else{
						activitydefinitionActivation = model.activityDefinitionWithName(arg1);
						activitydefinitionActivation2 = modelCh.activityDefinitionWithName(arg1);
					}
					
					if(!activityDefinitionsAlreadyAdded.contains(arg2)){
						activityDefinitionsAlreadyAdded.add(arg2);
						activitydefinitionTarget = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionTarget2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionTarget.setName(arg2);
						activitydefinitionTarget2.setName(arg2);
						activityID ++;
					}else{
						activitydefinitionTarget = model.activityDefinitionWithName(arg2);
						activitydefinitionTarget2 = modelCh.activityDefinitionWithName(arg2);
					}

					DataDeclareTemplate temp = getTemplate(CDarray[i]);					
					
					ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, DataDeclareTemplateConstraintTemplateMap.get(temp));
					org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID, modelCh, DataDeclareTemplateConstraintTemplateMap2.get(temp));
					
					Collection<Parameter> parameters = (DataDeclareTemplateConstraintTemplateMap.get(temp)).getParameters();
					Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = (DataDeclareTemplateConstraintTemplateMap2.get(temp)).getParameters();
					
					constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionActivation);
					constraintdefinition.addBranch((Parameter) parameters.toArray()[1], activitydefinitionTarget);
					
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionActivation2);
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[1], activitydefinitionTarget2);
					
					if(guards[i] != "") {
						String displaylabel = constraintdefinition.getDisplay();
						constraintdefinition.setDisplay(displaylabel + " " + guards[i]);
						constraintdefinition2.setDisplay(displaylabel + " " + guards[i]);
					}
					

					
					
					model.addConstraintDefiniton(constraintdefinition);
					modelCh.addConstraintDefiniton(constraintdefinition2);
					
					if(!activityDefinitionsToShow.contains(arg1)){
						activityDefinitionsToShow.add(arg1);
					}
					if(!activityDefinitionsToShow.contains(arg2)){
						activityDefinitionsToShow.add(arg2);
					}
					
					CDarray[i] = constraintdefinition;
					
					activityID++;
					constraintID++;
					

					break; }
				case Precedence: {	
					for(Parameter parameter : CDarray[i].getParameters()){
						if(CDarray[i].getBranches(parameter).iterator().hasNext()){
							String activityName = CDarray[i].getBranches(parameter).iterator().next().getName();
							if((activityName.contains("-assign")||activityName.contains("-ate_abort")||activityName.contains("-suspend")||activityName.contains("-complete")||activityName.contains("-autoskip")||activityName.contains("-manualskip")||activityName.contains("pi_abort")||activityName.contains("-reassign")||activityName.contains("-resume")||activityName.contains("-schedule")||activityName.contains("-start")||activityName.contains("-unknown")||activityName.contains("-withdraw"))){
								String[] splittedName = CDarray[i].getBranches(parameter).iterator().next().getName().split("-");
								activityName = splittedName[0];
								for(int j = 1; i<splittedName.length-1; i++){
									activityName = activityName + "-" + splittedName[j];
								}	
							}
							parameters2.add(activityName);
						}else{
							parameters2.add("EMPTY_PARAM");
						}
						p++;
						
					}
					
					arg1 = parameters2.get(p-1);
					arg2 = parameters2.get(p-2);
					
					if(!activityDefinitionsAlreadyAdded.contains(arg1)){
						activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionActivation.setName(arg1);
						activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionActivation2.setName(arg1);
						activityDefinitionsAlreadyAdded.add(arg1);
						activityID ++;
					}else{
						activitydefinitionActivation = model.activityDefinitionWithName(arg1);
						activitydefinitionActivation2 = modelCh.activityDefinitionWithName(arg1);
					}
					
					if(!activityDefinitionsAlreadyAdded.contains(arg2)){
						activityDefinitionsAlreadyAdded.add(arg2);
						activitydefinitionTarget = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionTarget.setName(arg2);
						activitydefinitionTarget2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionTarget2.setName(arg2);
						activityID ++;
					}else{
						activitydefinitionTarget = model.activityDefinitionWithName(arg2);
						activitydefinitionTarget2 = modelCh.activityDefinitionWithName(arg2);
					}
					
					DataDeclareTemplate temp = getTemplate(CDarray[i]);
					ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, DataDeclareTemplateConstraintTemplateMap.get(temp));
					org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID, modelCh, DataDeclareTemplateConstraintTemplateMap2.get(temp));
					
					Collection<Parameter> parameters = (DataDeclareTemplateConstraintTemplateMap.get(temp)).getParameters();
					Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = (DataDeclareTemplateConstraintTemplateMap2.get(temp)).getParameters();
					
					constraintdefinition.addBranch((Parameter) parameters.toArray()[1], activitydefinitionActivation);
					constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionTarget);
		
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[1], activitydefinitionActivation2);
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionTarget2);
		
					if(guards[i] != ""|| !guards[i].trim().isEmpty()) {
						String displaylabel = constraintdefinition.getDisplay();
						constraintdefinition.setDisplay(displaylabel + " " + guards[i]);
						constraintdefinition2.setDisplay(displaylabel + " " + guards[i]);
					}
		
					model.addConstraintDefiniton(constraintdefinition);
					modelCh.addConstraintDefiniton(constraintdefinition2);
					
					if(!activityDefinitionsToShow.contains(arg1)){
						activityDefinitionsToShow.add(arg1);
					}
					if(!activityDefinitionsToShow.contains(arg2)){
						activityDefinitionsToShow.add(arg2);
					}
					
					CDarray[i] = constraintdefinition;
					
					activityID++;
					constraintID++;
					break; }

			}	
			
		}	
		
		
		//Add the constraints which cannot be enriched with guards
		if(additional.length > 0){
			for(i = 0; i < additional.length; i++){
				if(additional[i].getParameters().size() < 2){
					for(Parameter parameter : additional[i].getParameters()){
						String activityName = additional[i].getBranches(parameter).iterator().next().getName();
						if(!activityDefinitionsAlreadyAdded.contains(activityName)){
							activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
							activitydefinitionActivation.setName(activityName);
							activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
							activitydefinitionActivation2.setName(activityName);
							activityDefinitionsAlreadyAdded.add(activityName);
							activityID ++;
						}else{
							activitydefinitionActivation = model.activityDefinitionWithName(activityName);
							activitydefinitionActivation2 = modelCh.activityDefinitionWithName(activityName);
						}
						String constraintName = additional[i].getName();
						String constraintDisplay = additional[i].getDisplay();
						String constraintDescription = additional[i].getDescription();
						String constraintText = additional[i].getText();
						org.processmining.plugins.declare.visualizing.Parameter param1 = new org.processmining.plugins.declare.visualizing.Parameter(0, activityName);
						Parameter param2 = new Parameter(0, activityName);
						param1.setBranchable(true);
						param2.setBranchable(true);
						org.processmining.plugins.declare.visualizing.ConstraintTemplate dct = new org.processmining.plugins.declare.visualizing.ConstraintTemplate(i,lang2);
						ConstraintTemplate dct2 = new ConstraintTemplate(i,lang);
						dct.addParameter(param1);
						dct.setDescription(constraintDescription);
						dct.setDisplay(constraintDisplay);
						dct.setName(constraintName);
						dct.setText(constraintText);
						
						dct2.addParameter(param2);
						dct2.setDescription(constraintDescription);
						dct2.setDisplay(constraintDisplay);
						dct2.setName(constraintName);
						dct2.setText(constraintText);
						
						org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID,modelCh,dct);
						ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, dct2);
						Collection<Parameter> parameters = dct2.getParameters();
						Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = dct.getParameters();
						
						constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionActivation);
						constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionActivation2);
						model.addConstraintDefiniton(constraintdefinition);
						modelCh.addConstraintDefiniton(constraintdefinition2);
						//System.out.println(constraintdefinition.getName() + " " + constraintdefinition2.getName());
						//System.out.println(((Parameter) parameters.toArray()[0]).getName() + " " + ((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0]).getName());
						if(!activityDefinitionsToShow.contains(activityName)){
							activityDefinitionsToShow.add(activityName);
						}
						activityID++;
						constraintID++;
					}
				}/*
				else {
					// Get the activities linked to the constraint
					for(Parameter parameter : additional[i].getParameters()){
						if(additional[i].getBranches(parameter).iterator().hasNext()){
							String activityName = additional[i].getBranches(parameter).iterator().next().getName();
							if((activityName.contains("-assign")||activityName.contains("-ate_abort")||activityName.contains("-suspend")||activityName.contains("-complete")||activityName.contains("-autoskip")||activityName.contains("-manualskip")||activityName.contains("pi_abort")||activityName.contains("-reassign")||activityName.contains("-resume")||activityName.contains("-schedule")||activityName.contains("-start")||activityName.contains("-unknown")||activityName.contains("-withdraw"))){
								String[] splittedName = additional[i].getBranches(parameter).iterator().next().getName().split("-");
								activityName = splittedName[0];
								for(int j = 1; i<splittedName.length-1; i++){
									activityName = activityName + "-" + splittedName[j];
								}	
							}
							parameters2.add(activityName);
						}else{
							parameters2.add("EMPTY_PARAM");
						}
						p++;
						
					}
					
					//The names of the activities are set correctly
					arg1 = parameters2.get(p-2);
					arg2 = parameters2.get(p-1);
					
					//Determine whether the activity is already present in the output model
					if(!activityDefinitionsAlreadyAdded.contains(arg1)){
						activitydefinitionActivation = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionActivation2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionActivation.setName(arg1);
						activitydefinitionActivation2.setName(arg1);
						activityDefinitionsAlreadyAdded.add(arg1);
						activityID ++;
					}else{
						activitydefinitionActivation = model.activityDefinitionWithName(arg1);
						activitydefinitionActivation2 = modelCh.activityDefinitionWithName(arg1);
					}
					
					if(!activityDefinitionsAlreadyAdded.contains(arg2)){
						activityDefinitionsAlreadyAdded.add(arg2);
						activitydefinitionTarget = model.addActivityDefinition(activityID); //new ActivityDefinition(parametersOfDiscoveredConstraintsInstantiationsOfCurrentTemplate[i][j], activityID, model);
						activitydefinitionTarget2 = modelCh.addActivityDefinition(activityID);
						activitydefinitionTarget.setName(arg2);
						activitydefinitionTarget2.setName(arg2);
						activityID ++;
					}else{
						activitydefinitionTarget = model.activityDefinitionWithName(arg2);
						activitydefinitionTarget2 = modelCh.activityDefinitionWithName(arg2);
					}
					String constraintName = additional[i].getName();
					String constraintDisplay = additional[i].getDisplay();
					String constraintDescription = additional[i].getDescription();
					String constraintText = additional[i].getText();
					
					org.processmining.plugins.declare.visualizing.Parameter param11 = new org.processmining.plugins.declare.visualizing.Parameter(0, arg1);
					org.processmining.plugins.declare.visualizing.Parameter param12 = new org.processmining.plugins.declare.visualizing.Parameter(1, arg2);
					Parameter param21 = new Parameter(0, arg1);
					Parameter param22 = new Parameter(1, arg2);
					param11.setBranchable(true);
					param21.setBranchable(true);
					param12.setBranchable(true);
					param22.setBranchable(true);
					
					org.processmining.plugins.declare.visualizing.ConstraintTemplate dct = new org.processmining.plugins.declare.visualizing.ConstraintTemplate(i,lang2);
					//org.processmining.plugins.declare.visualizing.ConstraintTemplate dct12 = new org.processmining.plugins.declare.visualizing.ConstraintTemplate(i+100,lang2);
					ConstraintTemplate dct2 = new ConstraintTemplate(i,lang);
					//ConstraintTemplate dct22 = new ConstraintTemplate(i+100,lang);
					
					dct.addParameter(param11);
					dct.addParameter(param12);
					dct.setDescription(constraintDescription);
					dct.setDisplay(constraintDisplay);
					dct.setName(constraintName);
					dct.setText(constraintText);
					
					dct2.addParameter(param21);
					dct2.addParameter(param22);
					dct2.setDescription(constraintDescription);
					dct2.setDisplay(constraintDisplay);
					dct2.setName(constraintName);
					dct2.setText(constraintText);
										
					ConstraintDefinition constraintdefinition = new ConstraintDefinition(constraintID, model, dct2);
					org.processmining.plugins.declare.visualizing.ConstraintDefinition constraintdefinition2 = new org.processmining.plugins.declare.visualizing.ConstraintDefinition(constraintID, modelCh, dct);
					
					Collection<Parameter> parameters = dct2.getParameters();
					Collection<org.processmining.plugins.declare.visualizing.Parameter> parameters3 = dct.getParameters();
					
					//Correctly set which activity is the activation
					constraintdefinition.addBranch((Parameter) parameters.toArray()[0], activitydefinitionActivation);
					constraintdefinition.addBranch((Parameter) parameters.toArray()[1], activitydefinitionTarget);
					
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[0], activitydefinitionActivation2);
					constraintdefinition2.addBranch((org.processmining.plugins.declare.visualizing.Parameter) parameters3.toArray()[1], activitydefinitionTarget2);
					
					//Add the new data constraint
					model.addConstraintDefiniton(constraintdefinition);
					modelCh.addConstraintDefiniton(constraintdefinition2);
					
					//Determine if the activity is already present in the model
					if(!activityDefinitionsToShow.contains(arg1)){
						activityDefinitionsToShow.add(arg1);
					}
					if(!activityDefinitionsToShow.contains(arg2)){
						activityDefinitionsToShow.add(arg2);
					}
					
					//Update the id's for the next iteration
					activityID++;
					constraintID++;
				}*/

			}
		}
		
		//Remove duplicate activities
		for(String ad : activityDefinitionsAlreadyAdded){
			if(!activityDefinitionsToShow.contains(ad)){
				model.deleteActivityDefinition(model.activityDefinitionWithName(ad));
			}
		}
		
		view = new AssignmentModelView(model);
		viewCh = new org.processmining.plugins.declare.visualizing.AssignmentModelView(modelCh);
		
		
		DGraph graph = view.getGraph();
		org.processmining.plugins.declare.visualizing.DGraph graph2 = viewCh.getGraph();
		
		final JGraphOrganicLayout oc = new JGraphOrganicLayout();
		
		oc.setDeterministic(true);
		oc.setOptimizeBorderLine(true);
		oc.setOptimizeEdgeCrossing(true);
		oc.setOptimizeEdgeDistance(true);
		oc.setOptimizeEdgeLength(true);
		oc.setOptimizeNodeDistribution(true);
		oc.setEdgeCrossingCostFactor(999999999);
		oc.setEdgeDistanceCostFactor(999999999);
		oc.setFineTuning(true);
		
		oc.setEdgeLengthCostFactor(9999);
		oc.setEdgeLengthCostFactor(99);
		oc.setNodeDistributionCostFactor(999999999);
		oc.setBorderLineCostFactor(999);
		oc.setRadiusScaleFactor(0.9);
		final JGraphFacade jgf = new JGraphFacade(graph);
		oc.run(jgf);
		final Map nestedMap = jgf.createNestedMap(true, true); 
		graph.getGraphLayoutCache().edit(nestedMap); 
	
		final JGraphFacade jgf2 = new JGraphFacade(graph2);
		oc.run(jgf2);
		final Map nestedMap2 = jgf2.createNestedMap(true, true); 
		graph2.getGraphLayoutCache().edit(nestedMap2); 
			
		map = new DataDeclareMap(model,view,null, modelCh, viewCh, null);
		map.setCDarray(CDarray);
		map.setCDNamearray(CDNamearray);
		map.setGuards(guards);
	    return map;
	}
	
	public Expression getGuard(int i) {
		return guardsExpression[i];
	}
	
	public void removeGuard(int i)
	{
		guardsExpression[i] = null;
	}
	
	public String getGuardAsString(int i) {
		if (guardsExpression[i]!=null) {
			return guardsExpression[i].toString();
		} else {
			return null;
		}
	}
	
	public void setGuardAsString(DataDeclareMap map, String guardAsString, int i) throws ParseException {
		setGuard(map, new Expression(guardAsString), i);
	}
	
	public void setGuard(DataDeclareMap map, Expression expr, int i){	
		String varName=containNonExistingVariable(map,expr);
		if (varName==null)
			throw new IllegalArgumentException("The expression is defined over an undefined variable "+varName);
		
		guardsExpression[i] = expr;
	}
	
	private String containNonExistingVariable(DataDeclareMap map,Expression operand) {
		Enumeration variableEnumeration=operand.findVariables();
		while(variableEnumeration.hasMoreElements())
		{
			String variableInExpression=((String) variableEnumeration.nextElement());
			if(map.hasVariable(variableInExpression.replace(PetriNetWithData.WRITE_CHAR, ' ').trim()))
				return variableInExpression;
		}
		return null;
	}
	
	
	
	private List<org.processmining.plugins.declare.visualizing.IItem> visit2(org.processmining.plugins.declare.visualizing.IItem item){
		List<org.processmining.plugins.declare.visualizing.IItem> templateList = new ArrayList<org.processmining.plugins.declare.visualizing.IItem>();
 		if (item instanceof org.processmining.plugins.declare.visualizing.LanguageGroup) {
 			org.processmining.plugins.declare.visualizing.LanguageGroup languageGroup = (org.processmining.plugins.declare.visualizing.LanguageGroup) item;
			List<org.processmining.plugins.declare.visualizing.IItem> childrenList = languageGroup.getChildren();
			for (org.processmining.plugins.declare.visualizing.IItem child : childrenList) {
				if (child instanceof org.processmining.plugins.declare.visualizing.LanguageGroup) {
					templateList.addAll(visit2(child));
				}else {
					templateList.add(child);
				}
			}
		}
		return templateList;
	}
	
	private List<IItem> visit(IItem item){
		List<IItem> templateList = new ArrayList<IItem>();
 		if (item instanceof LanguageGroup) {
			LanguageGroup languageGroup = (LanguageGroup) item;
			List<IItem> childrenList = languageGroup.getChildren();
			for (IItem child : childrenList) {
				if (child instanceof LanguageGroup) {
					templateList.addAll(visit(child));
				}else {
					templateList.add(child);
				}
			}
		}
		return templateList;
	}
	
	
	public static DataDeclareTemplate getTemplate(ConstraintDefinition constraint){
		if(constraint.getName().toLowerCase().equals("co-existence")){
			return DataDeclareTemplate.Co_Existence;
		}
		if(constraint.getName().toLowerCase().equals("not co-existence")){
			return DataDeclareTemplate.Not_Co_Existence;
		}
		if(constraint.getName().toLowerCase().equals("alternate precedence")){
			return DataDeclareTemplate.Alternate_Precedence;
		}
		if(constraint.getName().toLowerCase().equals("alternate response")){
			return DataDeclareTemplate.Alternate_Response;
		}
		
		if(constraint.getName().toLowerCase().equals("chain precedence")){
			return DataDeclareTemplate.Chain_Precedence;
		}
		if(constraint.getName().toLowerCase().equals("chain response")){
			return DataDeclareTemplate.Chain_Response;
		}
		
		if(constraint.getName().toLowerCase().equals("precedence")){
			return DataDeclareTemplate.Precedence;
		}
		if(constraint.getName().toLowerCase().equals("response")){
			return DataDeclareTemplate.Response;
		}
		
		if(constraint.getName().toLowerCase().equals("responded existence")){
			return DataDeclareTemplate.Responded_Existence;
		}
	
		if(constraint.getName().toLowerCase().equals("not chain response")){
			return DataDeclareTemplate.Not_Ch_Response;
		}
		if(constraint.getName().toLowerCase().equals("not precedence")){
			return DataDeclareTemplate.Not_Precedence;
		}	 
		if(constraint.getName().toLowerCase().equals("not chain precedence")){
			return DataDeclareTemplate.Not_Ch_Precedence;
		}
		if(constraint.getName().toLowerCase().equals("not response")){
			return DataDeclareTemplate.Not_Response;
		}	 
		if(constraint.getName().toLowerCase().equals("not responded existence")){
			return DataDeclareTemplate.Not_Responded_Existence;
		}
		
		if(constraint.getName().toLowerCase().equals("succession")){
			return DataDeclareTemplate.Succession;
		}
		
		if(constraint.getName().toLowerCase().equals("alternate succession")){
			return DataDeclareTemplate.Alternate_Succession;
		}
		
		if(constraint.getName().toLowerCase().equals("chain succession")){
			return DataDeclareTemplate.Chain_Succession;
		}
		
		if(constraint.getName().toLowerCase().equals("not succession")){
			return DataDeclareTemplate.Not_Succession;
		}
		
		if(constraint.getName().toLowerCase().equals("not chain succession")){
			return DataDeclareTemplate.Not_Ch_Succession;
		}
		
		
		return null;
	}
	

}
